#include <fintrf.h>
!
!     Matlab call : [v_ij v_ij_satellite time_watched time_watched_satellite outside_good_utility]=assemble_markets_fortran_mex(Pi_matrix,simulations_vector,marginal_distribution_parameters,rho,copula_parameter,market_demographics_draws,zero_out_draws,unobservable_draws,parametric_family_of_marginal_distributions,copula_parametric_family,X,satellite_channels,market_representative,dma_id,market_id,year_id,mu_eps)
!
!       Pi_matrix                                   (number_of_channels+1) x 7
!       simulations_vector                          1 x simulations
!       marginal_distribution_parameters            (number_of_channels+1) x 1
!       rho                                         (number_of_channels+1) x 8
!       copula_parameter                            (number_of_channels+1) x (number_of_channels+1)
!       market_demographics                         210 x 110 x simulations x 7
!       zero_out_draws                              (number_of_channels+1) x simulations x number_of_bundles
!       unobservable_draws                          (number_of_channels+1) x simulations x number_of_bundles
!       parametric_family_of_marginal_distributions scalar
!       copula_parametric_family                    scalar
!       X                                           number_of_bundles x (number_of_channels+1)
!       satellite_channels                          11 x number_of_channels
!       market_representative                       number_of_bundles x 1
!       dma_id                                      number_of_bundles x 1
!       market_id                                   number_of_bundles x 1
!       year_id                                     number_of_bundles x 1
!       mu_eps                                      scalar

!       Output
!       v_ij                                        number_of_bundles x simulations
!       v_ij_satellite                              number_of_bundles x simulations
!       time_watched                                number_of_channels x simulations x number_of_bundles
!       time_watched_satellite                      number_of_channels x simulations x number_of_bundles
!       outside_good_utility                        number_of_bundles x simulations

!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, time_watched_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer Pi_matrix,simulations_vector,marginal_distribution_parameters,rho,copula_parameter,market_demographics,zero_out,unobservable_draws,parametric_family_of_marginal_distributions,copula_parametric_family,X,satellite_channels,market_representative
      mwpointer dma_id, market_id, year_id, v_ij, v_ij_satellite, time_watched, time_watched_satellite, outside_good_utility, mu_eps
      mwsize mxgetm, mxgetn
      mwsize number_of_channels, number_of_bundles, simulations
    
!     RHS parameters
      
      Pi_matrix=mxgetpr(prhs(1)) 
      simulations_vector=  mxgetpr(prhs(2)) 
      marginal_distribution_parameters=mxgetpr(prhs(3)) 
      rho=mxgetpr(prhs(4)) 
      copula_parameter=mxgetpr(prhs(5)) 
      market_demographics=mxgetpr(prhs(6)) 
      zero_out=mxgetpr(prhs(7))  
      unobservable_draws=mxgetpr(prhs(8)) 
      parametric_family_of_marginal_distributions=mxgetpr(prhs(9)) 
      copula_parametric_family=mxgetpr(prhs(10))                    
      X=mxgetpr(prhs(11))                                           
      satellite_channels=mxgetpr(prhs(12))                                                         
      market_representative=mxgetpr(prhs(13))                       
      dma_id=mxgetpr(prhs(14))                       
      market_id=mxgetpr(prhs(15))            
      year_id=mxgetpr(prhs(16))           
      mu_eps=mxgetpr(prhs(17))   
    
        number_of_channels=mxgetm(prhs(1))
        simulations=mxgetn(prhs(2))
        number_of_bundles=mxgetm(prhs(15))

        time_watched_dimensions(1)=number_of_channels
        time_watched_dimensions(2)=simulations
        time_watched_dimensions(3)=number_of_bundles

      PLHS(1) = MXCREATEDOUBLEMATRIX(number_of_bundles,simulations,0)
      v_ij = MXGETPR(PLHS(1))
      PLHS(2) = MXCREATEDOUBLEMATRIX(number_of_bundles,simulations,0)
      v_ij_satellite = MXGETPR(PLHS(2))
      PLHS(3) = mxCreateNumericArray(3,time_watched_dimensions, mxClassIDFromClassName('double'), 0)
      time_watched = MXGETPR(PLHS(3))
      PLHS(4) = mxCreateNumericArray(3,time_watched_dimensions, mxClassIDFromClassName('double'), 0)
      time_watched_satellite = MXGETPR(PLHS(4))
      PLHS(5) = MXCREATEDOUBLEMATRIX(number_of_bundles,simulations,0)
      outside_good_utility = MXGETPR(PLHS(5))

      call assemble_markets_fortran_mex(%val(v_ij),%val(v_ij_satellite),%val(time_watched),%val(time_watched_satellite),%val(outside_good_utility),%val(marginal_distribution_parameters),%val(Pi_matrix),%val(copula_parameter),%val(rho),%val(unobservable_draws),%val(market_demographics),%val(zero_out),%val(dma_id),%val(market_id),%val(year_id),%val(X),%val(satellite_channels),%val(mu_eps),number_of_channels,simulations,number_of_bundles)
                         
      return
      end


        subroutine assemble_markets_fortran_mex(v_ij,v_ij_satellite,time_watched,time_watched_satellite,outside_good_utility,marginal_distribution_parameters,Pi_matrix,copula_parameter,rho,unobservable_draws,market_demographics,zero_out,dma_id,market_id,year_id,X,satellite_channels,mu_eps,number_of_channels,simulations,number_of_bundles)
        include 'link_fnl_static.h'
        !use imsl_libraries
        implicit none

        mwsize number_of_channels, simulations, number_of_bundles
        integer i, j, k, n
        double precision dma_id(number_of_bundles), market_id(number_of_bundles), year_id(number_of_bundles), adjusted_draws(number_of_channels,simulations), pre_interactions(number_of_channels,simulations), pre_unattributable_draws(number_of_channels,simulations)
        integer i_dma_id(number_of_bundles), i_market_id(number_of_bundles), i_year_id(number_of_bundles)
        double precision marginal_distribution_parameters(number_of_channels), Pi_matrix(number_of_channels,7), copula_parameter(number_of_channels,number_of_channels), X(number_of_bundles,number_of_channels), satellite_channels(11,number_of_channels-1)
        double precision unobs_draws_matrix(number_of_channels,simulations), pre_demographic_draw(number_of_channels,simulations), demographics_vector(7)
        double precision rho(number_of_channels,8), market_demographics(210,110,simulations,7),  zero_out(number_of_channels,simulations,number_of_bundles), unobservable_draws(number_of_channels,simulations,number_of_bundles)
        ! zero_out(number_of_channels,simulations,210,110), unobservable_draws(number_of_channels,simulations,210,110)
        double precision pre_gamma(number_of_channels,simulations), zero_this_out(number_of_channels,simulations), gamma_cable(number_of_channels,simulations), gamma_satellite(number_of_channels,simulations)
        double precision bundle_total_gamma(simulations), bundle_total_gamma_satellite(simulations)
        double precision v_ij(number_of_bundles,simulations), v_ij_satellite(number_of_bundles,simulations), outside_good_utility(number_of_bundles,simulations)
        double precision logged_utility(number_of_channels), logged_utility_satellite(number_of_channels)
        !double precision time_watched(number_of_bundles,simulations,number_of_channels), time_watched_satellite(number_of_bundles,simulations,number_of_channels), do_this
        double precision time_watched(number_of_channels,simulations,number_of_bundles), time_watched_satellite(number_of_channels,simulations,number_of_bundles), do_this
        double precision normal_cdf, mu_eps
        integer*4, external :: mexPrintf
        character(len=80) :: line
        INTEGER TID, OMP_GET_THREAD_NUM

        i_dma_id=INT(dma_id)
        i_market_id=INT(market_id)
        i_year_id=INT(year_id)

        ! for each market, bring in random draws and sum them up

!$OMP PARALLEL SHARED(unobservable_draws,copula_parameter,market_demographics,marginal_distribution_parameters,Pi_matrix,zero_out,rho,time_watched,time_watched_satellite,outside_good_utility,v_ij,v_ij_satellite) PRIVATE(j,k,unobs_draws_matrix,adjusted_draws,demographics_vector,pre_interactions,pre_unattributable_draws,pre_demographic_draw,zero_this_out,pre_gamma,gamma_cable,gamma_satellite,bundle_total_gamma,  bundle_total_gamma_satellite, logged_utility, logged_utility_satellite,TID), NUM_THREADS(12)

!$OMP DO SCHEDULE(DYNAMIC,3000)

       
        do i=1,number_of_bundles

            unobs_draws_matrix=unobservable_draws(:,:,i)
            adjusted_draws=matmul(copula_parameter,unobs_draws_matrix)

            do j=1,simulations
                 

                demographics_vector=market_demographics(i_dma_id(i),i_market_id(i),j,:)
                do k=1,number_of_channels
            
                   !pre_interactions(k,j)=ANORDF(adjusted_draws(k,j))
                    pre_interactions(k,j)=normal_cdf(adjusted_draws(k,j))

                    pre_unattributable_draws(k,j)=marginal_distribution_parameters(k)*-LOG(pre_interactions(k,j))
                    pre_demographic_draw(k,j)=DOT_PRODUCT(Pi_matrix(k,:),demographics_vector)   
                    
                    ! Mass points at zero. There are different rows for subsets of demographics (Black, Family, Older than 55)
                    ! Family, Black, Old
                    if ((demographics_vector(1).eq.1).AND.(demographics_vector(3).eq.1).AND.(demographics_vector(7).gt.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,1)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1
                        end if
                    ! Family, Non-Black, Old
                    elseif ((demographics_vector(1).eq.1).AND.(demographics_vector(3).eq.0).AND.(demographics_vector(7).gt.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,2)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Family, Non-Black, Not Old
                    elseif ((demographics_vector(1).eq.1).AND.(demographics_vector(3).eq.0).AND.(demographics_vector(7).le.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,3)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Family, Black, Not Old
                    elseif ((demographics_vector(1).eq.1).AND.(demographics_vector(3).eq.1).AND.(demographics_vector(7).le.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,4)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Non-Family, Black, Old                    
                    elseif ((demographics_vector(1).eq.0).AND.(demographics_vector(3).eq.1).AND.(demographics_vector(7).gt.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,5)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Non-Family, Non-Black, Old
                    elseif ((demographics_vector(1).eq.0).AND.(demographics_vector(3).eq.0).AND.(demographics_vector(7).gt.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,6)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Non-Family, Non-Black, Not Old
                    elseif ((demographics_vector(1).eq.0).AND.(demographics_vector(3).eq.0).AND.(demographics_vector(7).le.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,7)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    ! Non-Family, Black, Not Old
                    elseif ((demographics_vector(1).eq.0).AND.(demographics_vector(3).eq.1).AND.(demographics_vector(7).le.0.55D0)) then
                        if (zero_out(k,j,i).gt.rho(k,8)) then
                        zero_this_out(k,j)=0
                        else
                        zero_this_out(k,j)=1                    
                        end if
                    end if
                    


                    pre_gamma(k,j)=(pre_unattributable_draws(k,j)+pre_demographic_draw(k,j))*zero_this_out(k,j)

                    if (k.lt.number_of_channels) then
                    gamma_cable(k,j)=pre_gamma(k,j)*X(i,1+k)
                    gamma_satellite(k,j)=pre_gamma(k,j)*satellite_channels(-1996+i_year_id(i),k)
                    else
                    gamma_cable(k,j)=pre_gamma(k,j)
                    gamma_satellite(k,j)=pre_gamma(k,j)
                    end if

                end do

                bundle_total_gamma(j)=1D-7+SUM(gamma_cable(:,j))
                bundle_total_gamma_satellite(j)=1D-7+SUM(gamma_satellite(:,j))
           
        do k=1,number_of_channels

        time_watched(k,j,i)=gamma_cable(k,j)/bundle_total_gamma(j)
        time_watched_satellite(k,j,i)=gamma_satellite(k,j)/bundle_total_gamma_satellite(j)
        logged_utility(k)=pre_gamma(k,j)*LOG(1D0+(10080d0*time_watched(k,j,i)))
        logged_utility_satellite(k)=pre_gamma(k,j)*LOG(1D0+(10080d0*time_watched_satellite(k,j,i)))
        end do
      
        !outside_good_utility(i,j)=exp(pre_gamma(number_of_channels,j)*log(10081D0)*mu_eps)
        outside_good_utility(i,j)=1d0
        v_ij(i,j)=exp(SUM(logged_utility)*mu_eps-pre_gamma(number_of_channels,j)*log(10081D0)*mu_eps)
        !v_ij(i,j)=exp(SUM(logged_utility)*mu_eps)
        v_ij_satellite(i,j)=exp(SUM(logged_utility_satellite)*mu_eps-pre_gamma(number_of_channels,j)*log(10081D0)*mu_eps)
        !v_ij_satellite(i,j)=exp(SUM(logged_utility_satellite)*mu_eps)
        end do

        
        end do


!$OMP END DO NOWAIT

!$OMP END PARALLEL


           
        return
        end


        function normal_cdf(x)
        implicit none
        double precision normal_cdf, x, zz
        double precision p, c, b1, b2, b3, b4, b5

  !Abromowitz and Stegun approximation 
            b1 = 0.31938153
            b2 = -0.356563782
            b3 = 1.781477937
            b4 = -1.821255978
            b5 = 1.330274429
            p = 0.2316419
            c = 0.39894228

                   if (x.ge.0) then
                        zz=1/(1+p*x)
                        normal_cdf=(1 - c * exp(-x * x / 2) * zz * (zz * (zz * (zz * (zz * b5 + b4) + b3) + b2) + b1))
                   else
                        zz = 1/(1-p*x)
                        normal_cdf =(c * exp(-x * x / 2) * zz * (zz * (zz * (zz * (zz * b5 + b4) + b3) + b2) + b1));
                   end if

         return
         end function
