#include <fintrf.h>
!
!     Matlab call :  [player_profits profits market_share_big_market market_share_small_market consumer_buys_big_market consumer_buys_small_market upstream_fee_revenue]=bargaining_alc_downstream_firm_profits_mex(player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,channel_in_question);
!
!       player                                      scalar
!       p1                                          number_of_channels+1 x 1
!       p2                                          number_of_channels+1 x 1
!       p3                                          number_of_channels+1 x 1
!       p4                                          number_of_channels+1 x 1
!       bundles                                     number_of_channels x 4
!       beta_i_big_market                           simulations_big_market x 1
!       channels_utility_big_market                 simulations_big_market x number_of_channels 
!       beta_i_small_market                         simulations_small_market x 1
!       channels_utility_small_market               simulations_small_market x number_of_channels 
!       input_cost                                  number_of_channels x 4
!       logit_var                                   scalar
!       regulation_option                           scalar
!       market_size                                 2 x 1
!       compute_upstream_revenue                    scalar
!       utility_constants                           4 x 1
!       outside_good_utility                        simulations_big_market x 1
!       channel_in_question                         scalar

!       Output
!      	player_profits                              scalar
!      	profits                                     4 x 1
!       market_share_big_market                     simulations_big_market x 3
!       market_share_small_market                   simulations_small_market x 3
!       consumer_buys_big_market                    simulations_big_market x number_of_channels x 3
!       consumer_buys_small_market                  simulations_small_market x number_of_channels x 3
!       upstream_fee_revenue                        number_of_channels


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, channels_big_dimensions(3), channels_small_dimensions(3), mkt_share_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer player, p1, p2, p3, p4, beta_i_big_market, channels_utility_big_market, beta_i_small_market, channels_utility_small_market, input_cost, logit_var, regulation_option, market_size, bundles
      mwpointer profits, player_profits, market_share_big_market, market_share_small_market, consumer_buys_big_market, consumer_buys_small_market, upstream_fee_revenue, compute_upstream_revenue, utility_constants, outside_good_utility
      mwpointer channel_in_question
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels
    
!     RHS parameters
      
        player=mxgetpr(prhs(1)) 
        p1 =  mxgetpr(prhs(2)) 
        p2 =  mxgetpr(prhs(3)) 
        p3 =  mxgetpr(prhs(4)) 
        p4 =  mxgetpr(prhs(5)) 
        bundles = mxgetpr(prhs(6)) 
        beta_i_big_market = mxgetpr(prhs(7)) 
        channels_utility_big_market = mxgetpr(prhs(8)) 
        beta_i_small_market = mxgetpr(prhs(9)) 
        channels_utility_small_market = mxgetpr(prhs(10)) 
        input_cost = mxgetpr(prhs(11))
        logit_var=mxgetpr(prhs(12)) 
        regulation_option=mxgetpr(prhs(13))
        market_size=mxgetpr(prhs(14))
        compute_upstream_revenue=mxgetpr(prhs(15))
        utility_constants=mxgetpr(prhs(16))
        outside_good_utility=mxgetpr(prhs(17))
        channel_in_question=mxgetpr(prhs(18))

        simulations_big_market=mxgetm(prhs(7))
        simulations_small_market=mxgetm(prhs(9))
        number_of_channels=mxgetm(prhs(11))

        channels_big_dimensions(1)=simulations_big_market
        channels_big_dimensions(2)=number_of_channels
        channels_big_dimensions(3)=3
        channels_small_dimensions(1)=simulations_small_market
        channels_small_dimensions(2)=number_of_channels
        channels_small_dimensions(3)=3

        mkt_share_dimensions(1)=simulations_big_market
        mkt_share_dimensions(2)=3
        mkt_share_dimensions(3)=2

      PLHS(1) = mxcreatedoublescalar(0)
      player_profits = mxgetpr(PLHS(1))
      PLHS(2) = mxcreatedoublematrix(4,1,0)
      profits = mxgetpr(PLHS(2))
      PLHS(3) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_big_market = mxgetpr(PLHS(3))
      PLHS(4) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_small_market = mxgetpr(PLHS(4))
      PLHS(5) = mxCreateNumericArray(3,channels_big_dimensions, mxClassIDFromClassName('double'), 0)
      consumer_buys_big_market = mxgetpr(PLHS(5))
      PLHS(6) = mxCreateNumericArray(3,channels_small_dimensions, mxClassIDFromClassName('double'), 0)
      consumer_buys_small_market = mxgetpr(PLHS(6))
      PLHS(7) = mxcreatedoublematrix(number_of_channels,1,0)
      upstream_fee_revenue = mxgetpr(PLHS(7))

      call bargaining_alc_downstream_firm_profits_channel_mex(%val(player_profits),%val(profits),%val(market_share_big_market),%val(market_share_small_market),%val(consumer_buys_big_market),%val(consumer_buys_small_market),%val(upstream_fee_revenue),%val(player),%val(p1),%val(p2),%val(p3),%val(p4),%val(bundles),%val(beta_i_big_market),%val(channels_utility_big_market),%val(beta_i_small_market),%val(channels_utility_small_market),%val(input_cost),%val(logit_var),%val(regulation_option),%val(market_size),%val(compute_upstream_revenue),%val(utility_constants),%val(outside_good_utility),%val(channel_in_question),number_of_channels,simulations_big_market,simulations_small_market)
                   
      return
      end


        subroutine bargaining_alc_downstream_firm_profits_channel_mex(player_profits,profits,market_share_big_market,market_share_small_market,consumer_buys_big_market,consumer_buys_small_market,upstream_fee_revenue,player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,utility_constants,outside_good_utility,channel_in_question,number_of_channels,simulations_big_market,simulations_small_market)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels
        integer i, j, k, m, n, i_player 
        double precision profits(4), player, bundles(number_of_channels,4), p1(number_of_channels+1), p2(number_of_channels+1), p3(number_of_channels+1), p4(number_of_channels+1), input_cost(number_of_channels,4), logit_var, regulation_option
        double precision consumer_utility(simulations_big_market,3), consumer_utility_small(simulations_small_market,3), mu_eps, denominator(simulations_big_market), denominator_small(simulations_small_market), total_share
        double precision beta_i_big_market(simulations_big_market), channels_utility_big_market(simulations_big_market,number_of_channels), beta_i_small_market(simulations_small_market), channels_utility_small_market(simulations_small_market,number_of_channels)
        double precision bundle_utility_big_market(simulations_big_market), bundle_utility_small_market(simulations_small_market), market_share_big_market(simulations_big_market,3,2), market_share_small_market(simulations_small_market,3,2)
        double precision player_profits, market_size(2)
        double precision consumer_buys_big_market(simulations_big_market,number_of_channels,3), consumer_buys_small_market(simulations_small_market,number_of_channels,3)
        double precision profit_big_market(simulations_big_market,3), profit_small_market(simulations_small_market,3), compute_upstream_revenue
        double precision upstream_fee_revenue_big_market(simulations_big_market,number_of_channels), upstream_fee_revenue_small_market(simulations_small_market,number_of_channels), upstream_fee_revenue(number_of_channels), utility_constants(4), outside_good_utility(simulations_big_market)
        double precision nest_var, channel_in_question
        double precision cable_nest_utility_big_market(simulations_big_market,2), within_cable_denominator_big_market(simulations_big_market), within_cable_share_big_market(simulations_big_market,2)
        double precision cable_nest_utility_small_market(simulations_small_market,2), within_cable_denominator_small_market(simulations_small_market), within_cable_share_small_market(simulations_small_market,2)
        double precision directv_nest_utility_big_market(simulations_big_market,2), within_directv_denominator_big_market(simulations_big_market), within_directv_share_big_market(simulations_big_market,2)
        double precision directv_nest_utility_small_market(simulations_small_market,2), within_directv_denominator_small_market(simulations_small_market), within_directv_share_small_market(simulations_small_market,2)
        double precision dish_nest_utility_big_market(simulations_big_market,2), within_dish_denominator_big_market(simulations_big_market), within_dish_share_big_market(simulations_big_market,2)
        double precision dish_nest_utility_small_market(simulations_small_market,2), within_dish_denominator_small_market(simulations_small_market), within_dish_share_small_market(simulations_small_market,2)
        double precision nest_utility_big_market(simulations_big_market,3), cable_share_big_market(simulations_big_market), directv_share_big_market(simulations_big_market), dish_share_big_market(simulations_big_market)
        double precision nest_utility_small_market(simulations_small_market,3), cable_share_small_market(simulations_small_market), directv_share_small_market(simulations_small_market), dish_share_small_market(simulations_small_market)
        integer i_channel
        integer*4, external :: mexPrintf
        character(len=80) :: line
        
        nest_var=0.3d0;
        mu_eps=1d0/logit_var

        i_player=INT(player)
        i_channel=INT(channel_in_question)


if (regulation_option.eq.4d0) then
! ALC

!$OMP PARALLEL SHARED(channels_utility_big_market,beta_i_big_market,channels_utility_small_market,beta_i_small_market,bundles,p1,p2,p3,p4,consumer_buys_big_market,consumer_buys_small_market,cable_nest_utility_big_market, within_cable_denominator_big_market, within_cable_share_big_market, directv_nest_utility_big_market, within_directv_denominator_big_market, within_directv_share_big_market, dish_nest_utility_big_market, within_dish_denominator_big_market, within_dish_share_big_market, nest_utility_big_market, denominator, cable_share_big_market, directv_share_big_market, dish_share_big_market, market_share_big_market, cable_nest_utility_small_market, within_cable_denominator_small_market, within_cable_share_small_market, directv_nest_utility_small_market, within_directv_denominator_small_market, within_directv_share_small_market, dish_nest_utility_small_market, within_dish_denominator_small_market, within_dish_share_small_market, nest_utility_small_market, cable_share_small_market, directv_share_small_market, dish_share_small_market, market_share_small_market,mu_eps), NUM_THREADS(6)

!$OMP DO SCHEDULE(DYNAMIC,500)

            do j=1,simulations_big_market

            ! Find what channels each consumer buys

            do k=1,number_of_channels
            
            if (k.ne.i_channel) then

            if ((p1(k).le.(channels_utility_big_market(j,k)/(-beta_i_big_market(j)))).AND.(bundles(k,1).eq.1)) then
            consumer_buys_big_market(j,k,1)=1
            end if
            if ((p3(k).le.(channels_utility_big_market(j,k)/(-beta_i_big_market(j)))).AND.(bundles(k,3).eq.1)) then
            consumer_buys_big_market(j,k,2)=1
            end if
            if ((p4(k).le.(channels_utility_big_market(j,k)/(-beta_i_big_market(j)))).AND.(bundles(k,4).eq.1)) then
            consumer_buys_big_market(j,k,3)=1
            end if
            
            else 
            
            consumer_buys_big_market(j,k,1)=1*bundles(k,1)
            consumer_buys_big_market(j,k,2)=1*bundles(k,3)
            consumer_buys_big_market(j,k,3)=1*bundles(k,4)

            end if
            
            end do

            cable_nest_utility_big_market(j,1)= EXP( (-channels_utility_big_market(j,i_channel)*bundles(i_channel,1) - beta_i_big_market(j)*p1(i_channel)*bundles(i_channel,1) + DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,1)) + beta_i_big_market(j)*(DOT_PRODUCT(p1(1:number_of_channels),consumer_buys_big_market(j,:,1))))*(1/nest_var))
            cable_nest_utility_big_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,1)) + beta_i_big_market(j)*(DOT_PRODUCT(p1(1:number_of_channels),consumer_buys_big_market(j,:,1))))*(1/nest_var))
            
            within_cable_denominator_big_market(j) = SUM(cable_nest_utility_big_market(j,:))
            ! (1) is without channel, (2) with channel
            within_cable_share_big_market(j,1) = cable_nest_utility_big_market(j,1)/within_cable_denominator_big_market(j)
            within_cable_share_big_market(j,2) = cable_nest_utility_big_market(j,2)/within_cable_denominator_big_market(j)

            directv_nest_utility_big_market(j,1)= EXP( (-channels_utility_big_market(j,i_channel)*bundles(i_channel,3) - beta_i_big_market(j)*p3(i_channel)*bundles(i_channel,3) + DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,2)) + beta_i_big_market(j)*(DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_big_market(j,:,2))))*(1/nest_var))
            directv_nest_utility_big_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,2)) + beta_i_big_market(j)*(DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_big_market(j,:,2))))*(1/nest_var))

            within_directv_denominator_big_market(j) = SUM(directv_nest_utility_big_market(j,:))
            within_directv_share_big_market(j,1) = directv_nest_utility_big_market(j,1)/within_directv_denominator_big_market(j)
            within_directv_share_big_market(j,2) = directv_nest_utility_big_market(j,2)/within_directv_denominator_big_market(j)

            dish_nest_utility_big_market(j,1)= EXP( (-channels_utility_big_market(j,i_channel)*bundles(i_channel,4) - beta_i_big_market(j)*p4(i_channel)*bundles(i_channel,4) + DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,3)) + beta_i_big_market(j)*(DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_big_market(j,:,3))))*(1/nest_var))
            dish_nest_utility_big_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_big_market(j,:),consumer_buys_big_market(j,:,3)) + beta_i_big_market(j)*(DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_big_market(j,:,3))))*(1/nest_var))

            within_dish_denominator_big_market(j) = SUM(dish_nest_utility_big_market(j,:))
            within_dish_share_big_market(j,1) = dish_nest_utility_big_market(j,1)/within_dish_denominator_big_market(j)
            within_dish_share_big_market(j,2) = dish_nest_utility_big_market(j,2)/within_dish_denominator_big_market(j)

            nest_utility_big_market(j,1) = EXP((utility_constants(1) + beta_i_big_market(j)*p1(number_of_channels+1) + (nest_var)*LOG(within_cable_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,2) = EXP((utility_constants(3) + beta_i_big_market(j)*p3(number_of_channels+1) + (nest_var)*LOG(within_directv_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,3) = EXP((utility_constants(4) + beta_i_big_market(j)*p4(number_of_channels+1) + (nest_var)*LOG(within_dish_denominator_big_market(j)))*mu_eps)

            denominator(j) = outside_good_utility(j)+SUM(nest_utility_big_market(j,:))

            cable_share_big_market(j) = nest_utility_big_market(j,1)/denominator(j)
            directv_share_big_market(j) = nest_utility_big_market(j,2)/denominator(j)
            dish_share_big_market(j) = nest_utility_big_market(j,3)/denominator(j)

            market_share_big_market(j,1,1) = cable_share_big_market(j)*within_cable_share_big_market(j,1)
            market_share_big_market(j,1,2) = cable_share_big_market(j)*within_cable_share_big_market(j,2)
            market_share_big_market(j,2,1) = directv_share_big_market(j)*within_directv_share_big_market(j,1)
            market_share_big_market(j,2,2) = directv_share_big_market(j)*within_directv_share_big_market(j,2)
            market_share_big_market(j,3,1) = dish_share_big_market(j)*within_dish_share_big_market(j,1)
            market_share_big_market(j,3,2) = dish_share_big_market(j)*within_dish_share_big_market(j,2)

            end do

!$OMP END DO NOWAIT

!$OMP DO SCHEDULE(DYNAMIC,500)

            do j=1,simulations_small_market

            ! Find what channels each consumer buys

            do k=1,number_of_channels
            
            if (k.ne.i_channel) then

            if ((p2(k).le.(channels_utility_small_market(j,k)/(-beta_i_small_market(j)))).AND.(bundles(k,2).eq.1)) then
            consumer_buys_small_market(j,k,1)=1
            end if
            if ((p3(k).le.(channels_utility_small_market(j,k)/(-beta_i_small_market(j)))).AND.(bundles(k,3).eq.1)) then
            consumer_buys_small_market(j,k,2)=1
            end if
            if ((p4(k).le.(channels_utility_small_market(j,k)/(-beta_i_small_market(j)))).AND.(bundles(k,4).eq.1)) then
            consumer_buys_small_market(j,k,3)=1
            end if
            
            else 
            
            consumer_buys_small_market(j,k,1)=1*bundles(k,2)
            consumer_buys_small_market(j,k,2)=1*bundles(k,3)
            consumer_buys_small_market(j,k,3)=1*bundles(k,4)

            end if
            
            end do

            cable_nest_utility_small_market(j,1)= EXP( (-channels_utility_small_market(j,i_channel)*bundles(i_channel,2) - beta_i_small_market(j)*p2(i_channel)*bundles(i_channel,2) + DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,1)) + beta_i_small_market(j)*(DOT_PRODUCT(p2(1:number_of_channels),consumer_buys_small_market(j,:,1))))*(1/nest_var))
            cable_nest_utility_small_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,1)) + beta_i_small_market(j)*(DOT_PRODUCT(p2(1:number_of_channels),consumer_buys_small_market(j,:,1))))*(1/nest_var))
            
            within_cable_denominator_small_market(j) = SUM(cable_nest_utility_small_market(j,:))
            ! (1) is without channel, (2) with channel
            within_cable_share_small_market(j,1) = cable_nest_utility_small_market(j,1)/within_cable_denominator_small_market(j)
            within_cable_share_small_market(j,2) = cable_nest_utility_small_market(j,2)/within_cable_denominator_small_market(j)

            directv_nest_utility_small_market(j,1)= EXP( (-channels_utility_small_market(j,i_channel)*bundles(i_channel,3) - beta_i_small_market(j)*p3(i_channel)*bundles(i_channel,3) + DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,2)) + beta_i_small_market(j)*(DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_small_market(j,:,2))))*(1/nest_var))
            directv_nest_utility_small_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,2)) + beta_i_small_market(j)*(DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_small_market(j,:,2))))*(1/nest_var))

            within_directv_denominator_small_market(j) = SUM(directv_nest_utility_small_market(j,:))
            within_directv_share_small_market(j,1) = directv_nest_utility_small_market(j,1)/within_directv_denominator_small_market(j)
            within_directv_share_small_market(j,2) = directv_nest_utility_small_market(j,2)/within_directv_denominator_small_market(j)

            dish_nest_utility_small_market(j,1)= EXP( (-channels_utility_small_market(j,i_channel)*bundles(i_channel,4) - beta_i_small_market(j)*p4(i_channel)*bundles(i_channel,4) + DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,3)) + beta_i_small_market(j)*(DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_small_market(j,:,3))))*(1/nest_var))
            dish_nest_utility_small_market(j,2)= EXP( (DOT_PRODUCT(channels_utility_small_market(j,:),consumer_buys_small_market(j,:,3)) + beta_i_small_market(j)*(DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_small_market(j,:,3))))*(1/nest_var))

            within_dish_denominator_small_market(j) = SUM(dish_nest_utility_small_market(j,:))
            within_dish_share_small_market(j,1) = dish_nest_utility_small_market(j,1)/within_dish_denominator_small_market(j)
            within_dish_share_small_market(j,2) = dish_nest_utility_small_market(j,2)/within_dish_denominator_small_market(j)

            nest_utility_small_market(j,1) = EXP((utility_constants(2) + beta_i_small_market(j)*p2(number_of_channels+1) + (nest_var)*LOG(within_cable_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,2) = EXP((utility_constants(3) + beta_i_small_market(j)*p3(number_of_channels+1) + (nest_var)*LOG(within_directv_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,3) = EXP((utility_constants(4) + beta_i_small_market(j)*p4(number_of_channels+1) + (nest_var)*LOG(within_dish_denominator_small_market(j)))*mu_eps)

            denominator(j) = outside_good_utility(j)+SUM(nest_utility_small_market(j,:))

            cable_share_small_market(j) = nest_utility_small_market(j,1)/denominator(j)
            directv_share_small_market(j) = nest_utility_small_market(j,2)/denominator(j)
            dish_share_small_market(j) = nest_utility_small_market(j,3)/denominator(j)

            market_share_small_market(j,1,1) = cable_share_small_market(j)*within_cable_share_small_market(j,1)
            market_share_small_market(j,1,2) = cable_share_small_market(j)*within_cable_share_small_market(j,2)
            market_share_small_market(j,2,1) = directv_share_small_market(j)*within_directv_share_small_market(j,1)
            market_share_small_market(j,2,2) = directv_share_small_market(j)*within_directv_share_small_market(j,2)
            market_share_small_market(j,3,1) = dish_share_small_market(j)*within_dish_share_small_market(j,1)
            market_share_small_market(j,3,2) = dish_share_small_market(j)*within_dish_share_small_market(j,2)

            end do

!$OMP END DO NOWAIT

!$OMP END PARALLEL

            do j=1,simulations_big_market

            ! fix input payment savings
            profit_big_market(j,1) = (DOT_PRODUCT(p1(1:number_of_channels),consumer_buys_big_market(j,:,1)) - (p1(i_channel)-input_cost(i_channel,1))*bundles(i_channel,1) + p1(number_of_channels+1) - DOT_PRODUCT(input_cost(:,1),consumer_buys_big_market(j,:,1)))*market_share_big_market(j,1,1) + (DOT_PRODUCT(p1(1:number_of_channels),consumer_buys_big_market(j,:,1)) + p1(number_of_channels+1) - DOT_PRODUCT(input_cost(:,1),consumer_buys_big_market(j,:,1)))*market_share_big_market(j,1,2)
            profit_big_market(j,2) = (DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_big_market(j,:,2)) - (p3(i_channel)-input_cost(i_channel,3))*bundles(i_channel,3) + p3(number_of_channels+1) - DOT_PRODUCT(input_cost(:,3),consumer_buys_big_market(j,:,2)))*market_share_big_market(j,2,1) + (DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_big_market(j,:,2)) + p3(number_of_channels+1) - DOT_PRODUCT(input_cost(:,3),consumer_buys_big_market(j,:,2)))*market_share_big_market(j,2,2)
            profit_big_market(j,3) = (DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_big_market(j,:,3)) - (p4(i_channel)-input_cost(i_channel,4))*bundles(i_channel,4) + p4(number_of_channels+1) - DOT_PRODUCT(input_cost(:,4),consumer_buys_big_market(j,:,3)))*market_share_big_market(j,3,1) + (DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_big_market(j,:,3)) + p4(number_of_channels+1) - DOT_PRODUCT(input_cost(:,4),consumer_buys_big_market(j,:,3)))*market_share_big_market(j,3,2)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            if (k.ne.i_channel) then
            upstream_fee_revenue_big_market(j,k)=bundles(k,1)*input_cost(k,1)*consumer_buys_big_market(j,k,1)*(market_share_big_market(j,1,1)+market_share_big_market(j,1,2)) + bundles(k,3)*input_cost(k,3)*consumer_buys_big_market(j,k,2)*(market_share_big_market(j,2,1) + market_share_big_market(j,2,2)) + bundles(k,4)*input_cost(k,4)*consumer_buys_big_market(j,k,3)*(market_share_big_market(j,3,1) + market_share_big_market(j,3,2))
            elseif (k.eq.i_channel) then
            upstream_fee_revenue_big_market(j,k)=bundles(k,1)*input_cost(k,1)*consumer_buys_big_market(j,k,1)*(market_share_big_market(j,1,2)) + bundles(k,3)*input_cost(k,3)*consumer_buys_big_market(j,k,2)*(market_share_big_market(j,2,2)) + bundles(k,4)*input_cost(k,4)*consumer_buys_big_market(j,k,3)*(market_share_big_market(j,3,2))
            end if
            end do
            end if


            end do

            do j=1,simulations_small_market
    
            profit_small_market(j,1) = (DOT_PRODUCT(p2(1:number_of_channels),consumer_buys_small_market(j,:,1)) - (p2(i_channel)-input_cost(i_channel,2))*bundles(i_channel,2) + p2(number_of_channels+1) - DOT_PRODUCT(input_cost(:,2),consumer_buys_small_market(j,:,1)))*market_share_small_market(j,1,1) + (DOT_PRODUCT(p2(1:number_of_channels),consumer_buys_small_market(j,:,1)) + p2(number_of_channels+1) - DOT_PRODUCT(input_cost(:,2),consumer_buys_small_market(j,:,1)))*market_share_small_market(j,1,2)
            profit_small_market(j,2) = (DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_small_market(j,:,2)) - (p3(i_channel)-input_cost(i_channel,3))*bundles(i_channel,3) + p3(number_of_channels+1) - DOT_PRODUCT(input_cost(:,3),consumer_buys_small_market(j,:,2)))*market_share_small_market(j,2,1) + (DOT_PRODUCT(p3(1:number_of_channels),consumer_buys_small_market(j,:,2)) + p3(number_of_channels+1) - DOT_PRODUCT(input_cost(:,3),consumer_buys_small_market(j,:,2)))*market_share_small_market(j,2,2)
            profit_small_market(j,3) = (DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_small_market(j,:,3)) - (p4(i_channel)-input_cost(i_channel,4))*bundles(i_channel,4) + p4(number_of_channels+1) - DOT_PRODUCT(input_cost(:,4),consumer_buys_small_market(j,:,3)))*market_share_small_market(j,3,1) + (DOT_PRODUCT(p4(1:number_of_channels),consumer_buys_small_market(j,:,3)) + p4(number_of_channels+1) - DOT_PRODUCT(input_cost(:,4),consumer_buys_small_market(j,:,3)))*market_share_small_market(j,3,2)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            if (k.ne.i_channel) then
            upstream_fee_revenue_small_market(j,k)=bundles(k,2)*input_cost(k,2)*consumer_buys_small_market(j,k,1)*(market_share_small_market(j,1,1)+market_share_small_market(j,1,2)) + bundles(k,3)*input_cost(k,3)*consumer_buys_small_market(j,k,2)*(market_share_small_market(j,2,1) + market_share_small_market(j,2,2)) + bundles(k,4)*input_cost(k,4)*consumer_buys_small_market(j,k,3)*(market_share_small_market(j,3,1) + market_share_small_market(j,3,2))
            elseif (k.eq.i_channel) then
            upstream_fee_revenue_small_market(j,k)=bundles(k,2)*input_cost(k,2)*consumer_buys_small_market(j,k,1)*(market_share_small_market(j,1,2)) + bundles(k,3)*input_cost(k,3)*consumer_buys_small_market(j,k,2)*(market_share_small_market(j,2,2)) + bundles(k,4)*input_cost(k,4)*consumer_buys_small_market(j,k,3)*(market_share_small_market(j,3,2))
            end if
            end do
            end if


            end do

            profits(1) = sum(profit_big_market(:,1))*market_size(1)
            profits(2) = sum(profit_small_market(:,1))*market_size(2)
            profits(3) = sum(profit_big_market(:,2))*market_size(1) + sum(profit_small_market(:,2))*market_size(2)
            profits(4) = sum(profit_big_market(:,3))*market_size(1) + sum(profit_small_market(:,3))*market_size(2)

            player_profits=-profits(i_player)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue(k)=sum(upstream_fee_revenue_big_market(:,k))*market_size(1) + sum(upstream_fee_revenue_small_market(:,k))*market_size(2)
            end do
            end if

end if        

        return
        end

