#include <fintrf.h>
!
!     Matlab call :  [player_profits profits market_share_big_market market_share_small_market upstream_fee_revenue]=bargaining_bsp_downstream_firm_profits_mex(player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,utility_constants,market_size,compute_upstream_revenue,consumer_top_ten,channel_is_top_ten)
!
!       player                                      scalar
!       p1                                          2 x 1
!       p2                                          2 x 1
!       p3                                          2 x 1
!       p4                                          2 x 1
!       bundles                                     number_of_channels x 4
!       beta_i_big_market                           simulations_big_market x 1
!       channels_utility_big_market                 simulations_big_market x number_of_channels 
!       beta_i_small_market                         simulations_small_market x 1
!       channels_utility_small_market               simulations_small_market x number_of_channels 
!       input_cost                                  number_of_channels x 4
!       logit_var                                   scalar
!       utility_constants                           4x1
!       market_size                                 2 x 1
!       compute_upstream_revenue                    scalar
!       consumer_top_ten                            simulations_big_market x 10 x 4
!       channel_is_top_ten                          simulations_big_market x number_of_channels x 4

!       Output
!      	player_profits                              scalar
!      	profits                                     4 x 1
!       market_share_big_market                     simulations_big_market x 3 x 2 
!       market_share_small_market                   simulations_small_market x 3 x 2
!       upstream_fee_revenue                        number_of_channels


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, channels_big_dimensions(3), channels_small_dimensions(3), mkt_share_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer player, p1, p2, p3, p4, beta_i_big_market, channels_utility_big_market, beta_i_small_market, channels_utility_small_market, input_cost, logit_var, utility_constants, market_size, bundles
      mwpointer profits, player_profits, market_share_big_market, market_share_small_market, consumer_package_big_market, consumer_package_small_market, upstream_fee_revenue, compute_upstream_revenue
      mwpointer consumer_top_ten, channel_is_top_ten
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels
    
!     RHS parameters
      
        player=mxgetpr(prhs(1)) 
        p1 =  mxgetpr(prhs(2)) 
        p2 =  mxgetpr(prhs(3)) 
        p3 =  mxgetpr(prhs(4)) 
        p4 =  mxgetpr(prhs(5)) 
        bundles = mxgetpr(prhs(6)) 
        beta_i_big_market = mxgetpr(prhs(7)) 
        channels_utility_big_market = mxgetpr(prhs(8)) 
        beta_i_small_market = mxgetpr(prhs(9)) 
        channels_utility_small_market = mxgetpr(prhs(10)) 
        input_cost = mxgetpr(prhs(11))
        logit_var=mxgetpr(prhs(12)) 
        utility_constants=mxgetpr(prhs(13))
        market_size=mxgetpr(prhs(14))
        compute_upstream_revenue=mxgetpr(prhs(15))
        consumer_top_ten=mxgetpr(prhs(16))
        channel_is_top_ten=mxgetpr(prhs(17))

        simulations_big_market=mxgetm(prhs(7))
        simulations_small_market=mxgetm(prhs(9))
        number_of_channels=mxgetm(prhs(11))

        channels_big_dimensions(1)=simulations_big_market
        channels_big_dimensions(2)=15
        channels_big_dimensions(3)=3

        channels_small_dimensions(1)=simulations_small_market
        channels_small_dimensions(2)=15
        channels_small_dimensions(3)=3

        mkt_share_dimensions(1)=simulations_big_market
        mkt_share_dimensions(2)=2
        mkt_share_dimensions(3)=3

      PLHS(1) = mxcreatedoublescalar(0)
      player_profits = mxgetpr(PLHS(1))
      PLHS(2) = mxcreatedoublematrix(4,1,0)
      profits = mxgetpr(PLHS(2))
      PLHS(3) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_big_market = mxgetpr(PLHS(3))
      PLHS(4) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_small_market = mxgetpr(PLHS(4))
      PLHS(5) = mxcreatedoublematrix(number_of_channels,1,0)
      upstream_fee_revenue = mxgetpr(PLHS(5))

      call bargaining_bsp_downstream_firm_profits_mex(%val(player_profits),%val(profits),%val(market_share_big_market),%val(market_share_small_market),%val(upstream_fee_revenue),%val(player),%val(p1),%val(p2),%val(p3),%val(p4),%val(bundles),%val(beta_i_big_market),%val(channels_utility_big_market),%val(beta_i_small_market),%val(channels_utility_small_market),%val(input_cost),%val(logit_var),%val(utility_constants),%val(market_size),%val(compute_upstream_revenue),%val(consumer_top_ten),%val(channel_is_top_ten),number_of_channels,simulations_big_market,simulations_small_market)
                   
      return
      end


        subroutine bargaining_bsp_downstream_firm_profits_mex(player_profits,profits,market_share_big_market,market_share_small_market,upstream_fee_revenue,player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,utility_constants,market_size,compute_upstream_revenue,consumer_top_ten,channel_is_top_ten,number_of_channels,simulations_big_market,simulations_small_market)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels
        integer i, j, k, m, n, i_player 
        double precision profits(4), player, bundles(number_of_channels,4), p1(2), p2(2), p3(2), p4(2), input_cost(number_of_channels,4), logit_var, utility_constants(4)
        double precision beta_i_big_market(simulations_big_market), channels_utility_big_market(simulations_big_market,number_of_channels), beta_i_small_market(simulations_small_market), channels_utility_small_market(simulations_small_market,number_of_channels)
        double precision market_share_big_market(simulations_big_market,2,3), market_share_small_market(simulations_small_market,2,3)
        double precision player_profits, market_size(2), mu_eps, nest_var
        double precision pre_utility_big_market(simulations_big_market,2,3), pre_utility_small_market(simulations_small_market,2,3)
        double precision cable_denominator_big_market(simulations_big_market), cable_denominator_small_market(simulations_small_market)
        double precision within_cable_share_big_market(simulations_big_market,2), within_cable_share_small_market(simulations_small_market,2)
        double precision directv_denominator_big_market(simulations_big_market), directv_denominator_small_market(simulations_small_market)
        double precision within_directv_share_big_market(simulations_big_market,2), within_directv_share_small_market(simulations_small_market,2)
        double precision dish_denominator_big_market(simulations_big_market), dish_denominator_small_market(simulations_small_market)
        double precision within_dish_share_big_market(simulations_big_market,2), within_dish_share_small_market(simulations_small_market,2)
        double precision nest_utility_big_market(simulations_big_market,3), nest_utility_small_market(simulations_small_market,3)
        double precision denominator_big_market(simulations_big_market), denominator_small_market(simulations_small_market)
        double precision profit_big_market(simulations_big_market,3), profit_small_market(simulations_small_market,3), compute_upstream_revenue
        double precision upstream_fee_revenue_big_market(simulations_big_market,number_of_channels), upstream_fee_revenue_small_market(simulations_small_market,number_of_channels), upstream_fee_revenue(number_of_channels)
        double precision consumer_top_ten(simulations_big_market,15,4), channel_is_top_ten(simulations_big_market,number_of_channels,4)
        integer i_top_ten(simulations_big_market,15,4)
        integer*4, external :: mexPrintf
        character(len=80) :: line

        mu_eps=1d0/logit_var
        nest_var=0.3d0;
        i_player=INT(player)
        i_top_ten=INT(consumer_top_ten)


    !write(line,*) "price = ", p1
    !n = mexPrintf(line//achar(10)) 

! BSP


            do j=1,simulations_big_market

            ! Find which package each consumer buys
            ! sort channels before hand
            

            pre_utility_big_market(j,1,1)=EXP((1/nest_var)*(SUM(channels_utility_big_market(j,i_top_ten(j,:,1)))+beta_i_big_market(j)*p1(1)))
            pre_utility_big_market(j,2,1)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_big_market(j,:),bundles(:,1))+beta_i_big_market(j)*p1(2)))

            pre_utility_big_market(j,1,2)=EXP((1/nest_var)*(SUM(channels_utility_big_market(j,i_top_ten(j,:,3)))+beta_i_big_market(j)*p3(1)))
            pre_utility_big_market(j,2,2)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_big_market(j,:),bundles(:,3))+beta_i_big_market(j)*p3(2)))

            pre_utility_big_market(j,1,3)=EXP((1/nest_var)*(SUM(channels_utility_big_market(j,i_top_ten(j,:,4)))+beta_i_big_market(j)*p4(1)))
            pre_utility_big_market(j,2,3)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_big_market(j,:),bundles(:,4))+beta_i_big_market(j)*p4(2)))

            cable_denominator_big_market(j)=SUM( pre_utility_big_market(j,:,1) )
            within_cable_share_big_market(j,1) = pre_utility_big_market(j,1,1)/cable_denominator_big_market(j)
            within_cable_share_big_market(j,2) = pre_utility_big_market(j,2,1)/cable_denominator_big_market(j)

            directv_denominator_big_market(j)=SUM( pre_utility_big_market(j,:,2) )
            within_directv_share_big_market(j,1) = pre_utility_big_market(j,1,2)/directv_denominator_big_market(j)
            within_directv_share_big_market(j,2) = pre_utility_big_market(j,2,2)/directv_denominator_big_market(j)

            dish_denominator_big_market(j)=SUM( pre_utility_big_market(j,:,3) )
            within_dish_share_big_market(j,1) = pre_utility_big_market(j,1,3)/dish_denominator_big_market(j)
            within_dish_share_big_market(j,2) = pre_utility_big_market(j,2,3)/dish_denominator_big_market(j)

            nest_utility_big_market(j,1) = EXP((utility_constants(1) + (nest_var)*LOG(cable_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,2) = EXP((utility_constants(3) + (nest_var)*LOG(directv_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,3) = EXP((utility_constants(4) + (nest_var)*LOG(dish_denominator_big_market(j)))*mu_eps)

            denominator_big_market(j) = 1+SUM(nest_utility_big_market(j,:))

            market_share_big_market(j,1,1) = (nest_utility_big_market(j,1)/denominator_big_market(j))*within_cable_share_big_market(j,1)
            market_share_big_market(j,2,1) = (nest_utility_big_market(j,1)/denominator_big_market(j))*within_cable_share_big_market(j,2)
            market_share_big_market(j,1,2) = (nest_utility_big_market(j,2)/denominator_big_market(j))*within_directv_share_big_market(j,1)
            market_share_big_market(j,2,2) = (nest_utility_big_market(j,2)/denominator_big_market(j))*within_directv_share_big_market(j,2)
            market_share_big_market(j,1,3) = (nest_utility_big_market(j,3)/denominator_big_market(j))*within_dish_share_big_market(j,1)
            market_share_big_market(j,2,3) = (nest_utility_big_market(j,3)/denominator_big_market(j))*within_dish_share_big_market(j,2)

            end do

            do j=1,simulations_small_market

            ! Find which package each consumer buys
            ! sort channels before hand
            
            pre_utility_small_market(j,1,1)=EXP((1/nest_var)*(SUM(channels_utility_small_market(j,i_top_ten(j,:,2)))+beta_i_small_market(j)*p2(1)))
            pre_utility_small_market(j,2,1)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_small_market(j,:),bundles(:,2))+beta_i_small_market(j)*p2(2)))

            pre_utility_small_market(j,1,2)=EXP((1/nest_var)*(SUM(channels_utility_small_market(j,i_top_ten(j,:,3)))+beta_i_small_market(j)*p3(1)))
            pre_utility_small_market(j,2,2)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_small_market(j,:),bundles(:,3))+beta_i_small_market(j)*p3(2)))

            pre_utility_small_market(j,1,3)=EXP((1/nest_var)*(SUM(channels_utility_small_market(j,i_top_ten(j,:,4)))+beta_i_small_market(j)*p4(1)))
            pre_utility_small_market(j,2,3)=EXP((1/nest_var)*(DOT_PRODUCT(channels_utility_small_market(j,:),bundles(:,4))+beta_i_small_market(j)*p4(2)))

           
            cable_denominator_small_market(j)=SUM( pre_utility_small_market(j,:,1) )
            within_cable_share_small_market(j,1) = pre_utility_small_market(j,1,1)/cable_denominator_small_market(j)
            within_cable_share_small_market(j,2) = pre_utility_small_market(j,2,1)/cable_denominator_small_market(j)

            directv_denominator_small_market(j)=SUM( pre_utility_small_market(j,:,2) )
            within_directv_share_small_market(j,1) = pre_utility_small_market(j,1,2)/directv_denominator_small_market(j)
            within_directv_share_small_market(j,2) = pre_utility_small_market(j,2,2)/directv_denominator_small_market(j)

            dish_denominator_small_market(j)=SUM( pre_utility_small_market(j,:,3) )
            within_dish_share_small_market(j,1) = pre_utility_small_market(j,1,3)/dish_denominator_small_market(j)
            within_dish_share_small_market(j,2) = pre_utility_small_market(j,2,3)/dish_denominator_small_market(j)

            nest_utility_small_market(j,1) = EXP((utility_constants(2) + (nest_var)*LOG(cable_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,2) = EXP((utility_constants(3) + (nest_var)*LOG(directv_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,3) = EXP((utility_constants(4) + (nest_var)*LOG(dish_denominator_small_market(j)))*mu_eps)

            denominator_small_market(j) = 1+SUM(nest_utility_small_market(j,:))

            market_share_small_market(j,1,1) = (nest_utility_small_market(j,1)/denominator_small_market(j))*within_cable_share_small_market(j,1)
            market_share_small_market(j,2,1) = (nest_utility_small_market(j,1)/denominator_small_market(j))*within_cable_share_small_market(j,2)
            market_share_small_market(j,1,2) = (nest_utility_small_market(j,2)/denominator_small_market(j))*within_directv_share_small_market(j,1)
            market_share_small_market(j,2,2) = (nest_utility_small_market(j,2)/denominator_small_market(j))*within_directv_share_small_market(j,2)
            market_share_small_market(j,1,3) = (nest_utility_small_market(j,3)/denominator_small_market(j))*within_dish_share_small_market(j,1)
            market_share_small_market(j,2,3) = (nest_utility_small_market(j,3)/denominator_small_market(j))*within_dish_share_small_market(j,2)

            end do

            do j=1,simulations_big_market
    
            profit_big_market(j,1) = (p1(1)-SUM(input_cost(i_top_ten(j,:,1),1)))*market_share_big_market(j,1,1) + (p1(2)-DOT_PRODUCT(input_cost(:,1),bundles(:,1)))*market_share_big_market(j,2,1)
            profit_big_market(j,2) = (p3(1)-SUM(input_cost(i_top_ten(j,:,3),3)))*market_share_big_market(j,1,2) + (p3(2)-DOT_PRODUCT(input_cost(:,3),bundles(:,3)))*market_share_big_market(j,2,2)
            profit_big_market(j,3) = (p4(1)-SUM(input_cost(i_top_ten(j,:,4),4)))*market_share_big_market(j,1,3) + (p4(2)-DOT_PRODUCT(input_cost(:,4),bundles(:,4)))*market_share_big_market(j,2,3)
            
            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue_big_market(j,k)=bundles(k,1)*input_cost(k,1)*channel_is_top_ten(j,k,1)*market_share_big_market(j,1,1) + bundles(k,1)*input_cost(k,1)*market_share_big_market(j,2,1) + bundles(k,3)*input_cost(k,3)*channel_is_top_ten(j,k,3)*market_share_big_market(j,1,2) + bundles(k,3)*input_cost(k,3)*market_share_big_market(j,2,2)+ bundles(k,4)*input_cost(k,4)*channel_is_top_ten(j,k,4)*market_share_big_market(j,1,3) + bundles(k,4)*input_cost(k,4)*market_share_big_market(j,2,3)
            end do
            end if


            end do

            do j=1,simulations_small_market
    
            profit_small_market(j,1) = (p2(1)-SUM(input_cost(i_top_ten(j,:,2),2)))*market_share_small_market(j,1,1) + (p2(2)-DOT_PRODUCT(input_cost(:,2),bundles(:,2)))*market_share_small_market(j,2,1)
            profit_small_market(j,2) = (p3(1)-SUM(input_cost(i_top_ten(j,:,3),3)))*market_share_small_market(j,1,2) + (p3(2)-DOT_PRODUCT(input_cost(:,3),bundles(:,3)))*market_share_small_market(j,2,2)
            profit_small_market(j,3) = (p4(1)-SUM(input_cost(i_top_ten(j,:,4),4)))*market_share_small_market(j,1,3) + (p4(2)-DOT_PRODUCT(input_cost(:,4),bundles(:,4)))*market_share_small_market(j,2,3)
            
            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue_small_market(j,k)=bundles(k,2)*input_cost(k,2)*channel_is_top_ten(j,k,2)*market_share_small_market(j,1,1) + bundles(k,2)*input_cost(k,2)*market_share_small_market(j,2,1) + bundles(k,3)*input_cost(k,3)*channel_is_top_ten(j,k,3)*market_share_small_market(j,1,2) + bundles(k,3)*input_cost(k,3)*market_share_small_market(j,2,2)+ bundles(k,4)*input_cost(k,4)*channel_is_top_ten(j,k,4)*market_share_small_market(j,1,3) + bundles(k,4)*input_cost(k,4)*market_share_small_market(j,2,3)
            end do
            end if


            end do


            profits(1) = sum(profit_big_market(:,1))*market_size(1)
            profits(2) = sum(profit_small_market(:,1))*market_size(2)
            profits(3) = sum(profit_big_market(:,2))*market_size(1) + sum(profit_small_market(:,2))*market_size(2)
            profits(4) = sum(profit_big_market(:,3))*market_size(1) + sum(profit_small_market(:,3))*market_size(2)

            player_profits=-profits(i_player)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue(k)=sum(upstream_fee_revenue_big_market(:,k))*market_size(1) + sum(upstream_fee_revenue_small_market(:,k))*market_size(2)
            end do
            end if
 

        return
        end

