#include <fintrf.h>
!
!     Matlab call :  [player_profits profits market_share_big_market market_share_small_market upstream_fee_revenue]=bargaining_tt_downstream_firm_profits_mex(player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,utility_constants,outside_good_utility,in_package,packages);
!
!       player                                      scalar
!       p1                                          8 x 1
!       p2                                          8 x 1
!       p3                                          8 x 1
!       p4                                          8 x 1
!       bundles                                     number_of_channels x 4
!       beta_i_big_market                           simulations_big_market x 1
!       channels_utility_big_market                 simulations_big_market x number_of_channels 
!       beta_i_small_market                         simulations_small_market x 1
!       channels_utility_small_market               simulations_small_market x number_of_channels 
!       input_cost                                  number_of_channels x 4
!       logit_var                                   scalar
!       regulation_option                           scalar
!       market_size                                 2 x 1
!       compute_upstream_revenue                    scalar
!       utility_constants                           4 x 1
!       outside_good_utility                        simulations_big_market x 1
!       in_package                                  number_of_packages x number_of_channels
!       packages                                    number_of_packages x 7

!       Output
!      	player_profits                              scalar
!      	profits                                     4 x 1
!       market_share_big_market                     simulations_big_market x number_of_packages x 3
!       market_share_small_market                   simulations_small_market x number_of_packages x 3
!       upstream_fee_revenue                        number_of_channels


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, mkt_share_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer player, p1, p2, p3, p4, beta_i_big_market, channels_utility_big_market, beta_i_small_market, channels_utility_small_market, input_cost, logit_var, regulation_option, market_size, bundles
      mwpointer profits, player_profits, market_share_big_market, market_share_small_market, upstream_fee_revenue, compute_upstream_revenue, utility_constants, outside_good_utility
      mwpointer package_utility_big_market,within_cable_denominator_big_market,within_directv_denominator_big_market,within_dish_denominator_big_market,within_cable_share_big_market,within_directv_share_big_market,within_dish_share_big_market,package_utility_small_market,within_cable_denominator_small_market,within_directv_denominator_small_market,within_dish_denominator_small_market,within_cable_share_small_market,within_directv_share_small_market,within_dish_share_small_market
      mwpointer in_package, packages
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels, number_of_packages
    
!     RHS parameters
      
        player=mxgetpr(prhs(1)) 
        p1 =  mxgetpr(prhs(2)) 
        p2 =  mxgetpr(prhs(3)) 
        p3 =  mxgetpr(prhs(4)) 
        p4 =  mxgetpr(prhs(5)) 
        bundles = mxgetpr(prhs(6)) 
        beta_i_big_market = mxgetpr(prhs(7)) 
        channels_utility_big_market = mxgetpr(prhs(8)) 
        beta_i_small_market = mxgetpr(prhs(9)) 
        channels_utility_small_market = mxgetpr(prhs(10)) 
        input_cost = mxgetpr(prhs(11))
        logit_var=mxgetpr(prhs(12)) 
        regulation_option=mxgetpr(prhs(13))
        market_size=mxgetpr(prhs(14))
        compute_upstream_revenue=mxgetpr(prhs(15))
        utility_constants=mxgetpr(prhs(16))
        outside_good_utility=mxgetpr(prhs(17))
        in_package=mxgetpr(prhs(18))
        packages=mxgetpr(prhs(19))
        package_utility_big_market=mxgetpr(prhs(20))
        within_cable_denominator_big_market=mxgetpr(prhs(21))
        within_directv_denominator_big_market=mxgetpr(prhs(22))
        within_dish_denominator_big_market=mxgetpr(prhs(23))
        within_cable_share_big_market=mxgetpr(prhs(24))
        within_directv_share_big_market=mxgetpr(prhs(25))
        within_dish_share_big_market=mxgetpr(prhs(26))
        package_utility_small_market=mxgetpr(prhs(27))
        within_cable_denominator_small_market=mxgetpr(prhs(28))
        within_directv_denominator_small_market=mxgetpr(prhs(29))
        within_dish_denominator_small_market=mxgetpr(prhs(30))
        within_cable_share_small_market=mxgetpr(prhs(31))
        within_directv_share_small_market=mxgetpr(prhs(32))
        within_dish_share_small_market=mxgetpr(prhs(33))

        simulations_big_market=mxgetm(prhs(7))
        simulations_small_market=mxgetm(prhs(9))
        number_of_channels=mxgetm(prhs(11))
        number_of_packages=mxgetm(prhs(18))

        mkt_share_dimensions(1)=simulations_big_market
        mkt_share_dimensions(2)=number_of_packages
        mkt_share_dimensions(3)=3

      PLHS(1) = mxcreatedoublescalar(0)
      player_profits = mxgetpr(PLHS(1))
      PLHS(2) = mxcreatedoublematrix(4,1,0)
      profits = mxgetpr(PLHS(2))
      PLHS(3) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_big_market = mxgetpr(PLHS(3))
      PLHS(4) = mxCreateNumericArray(3,mkt_share_dimensions, mxClassIDFromClassName('double'), 0)
      market_share_small_market = mxgetpr(PLHS(4))
      PLHS(5) = mxcreatedoublematrix(number_of_channels,1,0)
      upstream_fee_revenue = mxgetpr(PLHS(5))

      call bargaining_tt_downstream_firm_profits_fast_mex(%val(player_profits),%val(profits),%val(market_share_big_market),%val(market_share_small_market),%val(upstream_fee_revenue),%val(player),%val(p1),%val(p2),%val(p3),%val(p4),%val(bundles),%val(beta_i_big_market),%val(channels_utility_big_market),%val(beta_i_small_market),%val(channels_utility_small_market),%val(input_cost),%val(logit_var),%val(regulation_option),%val(market_size),%val(compute_upstream_revenue),%val(utility_constants),%val(outside_good_utility),%val(in_package),%val(packages),%val(package_utility_big_market),%val(within_cable_denominator_big_market),%val(within_directv_denominator_big_market),%val(within_dish_denominator_big_market),%val(within_cable_share_big_market),%val(within_directv_share_big_market),%val(within_dish_share_big_market),%val(package_utility_small_market),%val(within_cable_denominator_small_market),%val(within_directv_denominator_small_market),%val(within_dish_denominator_small_market),%val(within_cable_share_small_market),%val(within_directv_share_small_market),%val(within_dish_share_small_market),number_of_channels,simulations_big_market,simulations_small_market,number_of_packages)
                   
      return
      end


        subroutine bargaining_tt_downstream_firm_profits_fast_mex(player_profits,profits,market_share_big_market,market_share_small_market,upstream_fee_revenue,player,p1,p2,p3,p4,bundles,beta_i_big_market,channels_utility_big_market,beta_i_small_market,channels_utility_small_market,input_cost,logit_var,regulation_option,market_size,compute_upstream_revenue,utility_constants,outside_good_utility,in_package,packages,package_utility_big_market,within_cable_denominator_big_market,within_directv_denominator_big_market,within_dish_denominator_big_market,within_cable_share_big_market,within_directv_share_big_market,within_dish_share_big_market,package_utility_small_market,within_cable_denominator_small_market,within_directv_denominator_small_market,within_dish_denominator_small_market,within_cable_share_small_market,within_directv_share_small_market,within_dish_share_small_market,number_of_channels,simulations_big_market,simulations_small_market,number_of_packages)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels, number_of_packages
        integer i, j, k, m, n, i_player 
        double precision profits(4), player, bundles(number_of_channels,4), p1(6), p2(6), p3(6), p4(6), input_cost(number_of_channels,4), logit_var, regulation_option
        double precision consumer_utility(simulations_big_market,3), consumer_utility_small(simulations_small_market,3), mu_eps, denominator(simulations_big_market), denominator_small(simulations_small_market), total_share
        double precision beta_i_big_market(simulations_big_market), channels_utility_big_market(simulations_big_market,number_of_channels), beta_i_small_market(simulations_small_market), channels_utility_small_market(simulations_small_market,number_of_channels)
        double precision bundle_utility_big_market(simulations_big_market), bundle_utility_small_market(simulations_small_market)
        double precision player_profits, market_size(2)
        double precision consumer_buys_big_market(simulations_big_market,number_of_channels,3), consumer_buys_small_market(simulations_small_market,number_of_channels,3)
        double precision compute_upstream_revenue
        double precision upstream_fee_revenue(number_of_channels), utility_constants(4), outside_good_utility(simulations_big_market)
        double precision nest_var
        double precision package_utility_big_market(simulations_big_market,number_of_packages,3), package_utility_small_market(simulations_small_market,number_of_packages,3)
        double precision cable_share_big_market(simulations_big_market), directv_share_big_market(simulations_big_market), dish_share_big_market(simulations_big_market)
        double precision cable_share_small_market(simulations_big_market), directv_share_small_market(simulations_big_market), dish_share_small_market(simulations_big_market)
        double precision within_cable_denominator_big_market(simulations_big_market), within_cable_share_big_market(simulations_big_market,number_of_packages)
        double precision within_directv_denominator_big_market(simulations_big_market), within_directv_share_big_market(simulations_big_market,number_of_packages)
        double precision within_dish_denominator_big_market(simulations_big_market), within_dish_share_big_market(simulations_big_market,number_of_packages)
        double precision within_cable_denominator_small_market(simulations_small_market), within_cable_share_small_market(simulations_small_market,number_of_packages)
        double precision within_directv_denominator_small_market(simulations_small_market), within_directv_share_small_market(simulations_small_market,number_of_packages)
        double precision within_dish_denominator_small_market(simulations_small_market), within_dish_share_small_market(simulations_small_market,number_of_packages)
        double precision nest_utility_big_market(simulations_big_market,3), market_share_big_market(simulations_big_market,number_of_packages,3)
        double precision nest_utility_small_market(simulations_small_market,3), market_share_small_market(simulations_small_market,number_of_packages,3)
        double precision pre_profit_big_market(simulations_big_market,number_of_packages,3), pre_profit_small_market(simulations_small_market,number_of_packages,3)
        double precision profit_big_market(simulations_big_market,3), profit_small_market(simulations_small_market,3)
        double precision pre_upstream_fee_revenue_big_market(simulations_big_market,number_of_channels,number_of_packages), upstream_fee_revenue_big_market(simulations_big_market,number_of_channels)
        double precision pre_upstream_fee_revenue_small_market(simulations_small_market,number_of_channels,number_of_packages), upstream_fee_revenue_small_market(simulations_small_market,number_of_channels)
        double precision in_package(number_of_packages,number_of_channels), packages(number_of_packages,5)
        double precision p1_package(number_of_packages), p2_package(number_of_packages), p3_package(number_of_packages), p4_package(number_of_packages)
        double precision pre_package_cost(number_of_packages,number_of_channels,4), package_cost(number_of_packages,4)
        integer*4, external :: mexPrintf
        character(len=80) :: line
        
        nest_var=1d0;
        mu_eps=1d0/logit_var

        i_player=INT(player)



! N themed tiers --> 2^N - 1 packages
do k=1,number_of_packages

p1_package(k)=DOT_PRODUCT(p1(1:5),packages(k,:))
p2_package(k)=DOT_PRODUCT(p2(1:5),packages(k,:))
p3_package(k)=DOT_PRODUCT(p3(1:5),packages(k,:))
p4_package(k)=DOT_PRODUCT(p4(1:5),packages(k,:))

end do


do k=1,number_of_packages
do j=1,number_of_channels
pre_package_cost(k,j,1)=input_cost(j,1)*bundles(j,1)*in_package(k,j)
pre_package_cost(k,j,2)=input_cost(j,2)*bundles(j,2)*in_package(k,j)
pre_package_cost(k,j,3)=input_cost(j,3)*bundles(j,3)*in_package(k,j)
pre_package_cost(k,j,4)=input_cost(j,4)*bundles(j,4)*in_package(k,j)
end do
package_cost(k,1)=sum(pre_package_cost(k,:,1))
package_cost(k,2)=sum(pre_package_cost(k,:,2))
package_cost(k,3)=sum(pre_package_cost(k,:,3))
package_cost(k,4)=sum(pre_package_cost(k,:,4))
end do

! Themed Tiers
    !write(line,*) "price = ", p1_package(1)
    !n = mexPrintf(line//achar(10)) 


            do j=1,simulations_big_market

            ! Find utility for each package from each distributor

            nest_utility_big_market(j,1) = EXP((utility_constants(1) + beta_i_big_market(j)*p1(6) + (nest_var)*LOG(within_cable_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,2) = EXP((utility_constants(3) + beta_i_big_market(j)*p3(6) + (nest_var)*LOG(within_directv_denominator_big_market(j)))*mu_eps)
            nest_utility_big_market(j,3) = EXP((utility_constants(4) + beta_i_big_market(j)*p4(6) + (nest_var)*LOG(within_dish_denominator_big_market(j)))*mu_eps)

            denominator(j) = outside_good_utility(j)+SUM(nest_utility_big_market(j,:))

            cable_share_big_market(j) = nest_utility_big_market(j,1)/denominator(j)
            directv_share_big_market(j) = nest_utility_big_market(j,2)/denominator(j)
            dish_share_big_market(j) = nest_utility_big_market(j,3)/denominator(j)

            do k=1,number_of_packages
            market_share_big_market(j,k,1) = cable_share_big_market(j)*within_cable_share_big_market(j,k)
            market_share_big_market(j,k,2) = directv_share_big_market(j)*within_directv_share_big_market(j,k)
            market_share_big_market(j,k,3) = dish_share_big_market(j)*within_dish_share_big_market(j,k)
            end do

            end do


             do j=1,simulations_small_market

            ! Find utility for each package from each distributor

            nest_utility_small_market(j,1) = EXP((utility_constants(2) + beta_i_small_market(j)*p2(6) + (nest_var)*LOG(within_cable_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,2) = EXP((utility_constants(3) + beta_i_small_market(j)*p3(6) + (nest_var)*LOG(within_directv_denominator_small_market(j)))*mu_eps)
            nest_utility_small_market(j,3) = EXP((utility_constants(4) + beta_i_small_market(j)*p4(6) + (nest_var)*LOG(within_dish_denominator_small_market(j)))*mu_eps)

            denominator(j) = outside_good_utility(j)+SUM(nest_utility_small_market(j,:))

            cable_share_small_market(j) = nest_utility_small_market(j,1)/denominator(j)
            directv_share_small_market(j) = nest_utility_small_market(j,2)/denominator(j)
            dish_share_small_market(j) = nest_utility_small_market(j,3)/denominator(j)

            do k=1,number_of_packages
            market_share_small_market(j,k,1) = cable_share_small_market(j)*within_cable_share_small_market(j,k)
            market_share_small_market(j,k,2) = directv_share_small_market(j)*within_directv_share_small_market(j,k)
            market_share_small_market(j,k,3) = dish_share_small_market(j)*within_dish_share_small_market(j,k)
            end do

            end do


            do j=1,simulations_big_market

            do k=1,number_of_packages
            pre_profit_big_market(j,k,1) = market_share_big_market(j,k,1)*(p1(6) + p1_package(k) - SUM(input_cost(:,1)*bundles(:,1)*in_package(k,:)))
            pre_profit_big_market(j,k,2) = market_share_big_market(j,k,2)*(p3(6) + p3_package(k) - SUM(input_cost(:,3)*bundles(:,3)*in_package(k,:)))
            pre_profit_big_market(j,k,3) = market_share_big_market(j,k,3)*(p4(6) + p4_package(k) - SUM(input_cost(:,4)*bundles(:,4)*in_package(k,:)))
            end do
            
            profit_big_market(j,1)=SUM(pre_profit_big_market(j,:,1))
            profit_big_market(j,2)=SUM(pre_profit_big_market(j,:,2))
            profit_big_market(j,3)=SUM(pre_profit_big_market(j,:,3))

            if (compute_upstream_revenue.eq.1) then

            do i=1,number_of_channels
            do k=1,number_of_packages

            pre_upstream_fee_revenue_big_market(j,i,k)=in_package(k,i)*(bundles(i,1)*input_cost(i,1)*market_share_big_market(j,k,1) + bundles(i,3)*input_cost(i,3)*market_share_big_market(j,k,2) + bundles(i,4)*input_cost(i,4)*market_share_big_market(j,k,3))
            
            end do
                upstream_fee_revenue_big_market(j,i)=SUM(pre_upstream_fee_revenue_big_market(j,i,:))

            end do

            end if
            end do

             do j=1,simulations_small_market

            do k=1,number_of_packages
            pre_profit_small_market(j,k,1) = market_share_small_market(j,k,1)*(p2(6) + p2_package(k) - SUM(input_cost(:,2)*bundles(:,2)*in_package(k,:)))
            pre_profit_small_market(j,k,2) = market_share_small_market(j,k,2)*(p3(6) + p3_package(k) - SUM(input_cost(:,3)*bundles(:,3)*in_package(k,:)))
            pre_profit_small_market(j,k,3) = market_share_small_market(j,k,3)*(p4(6) + p4_package(k) - SUM(input_cost(:,4)*bundles(:,4)*in_package(k,:)))
            end do

            profit_small_market(j,1)=SUM(pre_profit_small_market(j,:,1))
            profit_small_market(j,2)=SUM(pre_profit_small_market(j,:,2))
            profit_small_market(j,3)=SUM(pre_profit_small_market(j,:,3))

            if (compute_upstream_revenue.eq.1) then

            do i=1,number_of_channels
            do k=1,number_of_packages

            pre_upstream_fee_revenue_small_market(j,i,k)=in_package(k,i)*(bundles(i,2)*input_cost(i,2)*market_share_small_market(j,k,1) + bundles(i,3)*input_cost(i,3)*market_share_small_market(j,k,2) + bundles(i,4)*input_cost(i,4)*market_share_small_market(j,k,3))
            
            end do
                upstream_fee_revenue_small_market(j,i)=SUM(pre_upstream_fee_revenue_small_market(j,i,:))

            end do

            end if
            end do


            profits(1) = sum(profit_big_market(:,1))*market_size(1)
            profits(2) = sum(profit_small_market(:,1))*market_size(2)
            profits(3) = sum(profit_big_market(:,2))*market_size(1) + sum(profit_small_market(:,2))*market_size(2)
            profits(4) = sum(profit_big_market(:,3))*market_size(1) + sum(profit_small_market(:,3))*market_size(2)

            player_profits=-profits(i_player)

            if (compute_upstream_revenue.eq.1) then
            do k=1,number_of_channels
            upstream_fee_revenue(k)=SUM(upstream_fee_revenue_big_market(:,k))*market_size(1) + SUM(upstream_fee_revenue_small_market(:,k))*market_size(2)
            end do
            end if


        return
        end

