#include <fintrf.h>
!
!     Matlab call :  time_watched=bargaining_tt_time_watched(bundles,market_share_big_market,market_share_small_market,pre_gamma_big_market,pre_gamma_small_market,market_size,in_package,packages);
!
!       bundles                                     number_of_channels x 4
!       market_share_big_market                     simulations_big_market x number_of_packages x 3
!       market_share_small_market                   simulations_small_market x number_of_packages x 3
!       pre_gamma_big_market                        (number_of_channels+1) x simulations_big_market
!       pre_gamma_small_market                      (number_of_channels+1) x simulations_small_market
!       market_size                                 2 x 1
!       in_package                                  number_of_packages x number_of_channels
!       packages                                    number_of_packages x 7

!       Output
!       time_watched_big_market                     number_of_channels x simulations_big_market
!       time_watched_small_market                   number_of_channels x simulations_small_market


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs, mkt_share_dimensions(3)
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer market_size, bundles, pre_gamma_big_market, pre_gamma_small_market
      mwpointer market_share_big_market, market_share_small_market, time_watched_big_market, time_watched_small_market
      mwpointer in_package, packages
      mwsize mxgetm, mxgetn
      mwsize simulations_big_market, simulations_small_market, number_of_channels, number_of_packages
    
!     RHS parameters
      
        bundles=mxgetpr(prhs(1)) 
        market_share_big_market =  mxgetpr(prhs(2)) 
        market_share_small_market =  mxgetpr(prhs(3)) 
        pre_gamma_big_market =  mxgetpr(prhs(4)) 
        pre_gamma_small_market =  mxgetpr(prhs(5)) 
        market_size = mxgetpr(prhs(6)) 
        in_package=mxgetpr(prhs(7))
        packages=mxgetpr(prhs(8))

        simulations_big_market=mxgetm(prhs(4))
        simulations_small_market=mxgetm(prhs(5))
        number_of_channels=mxgetn(prhs(7))
        number_of_packages=mxgetm(prhs(7))


      PLHS(1) = mxcreatedoublematrix(simulations_big_market,number_of_channels,0)
      time_watched_big_market  = mxgetpr(PLHS(1))
      PLHS(2) = mxcreatedoublematrix(simulations_small_market,number_of_channels,0)
      time_watched_small_market  = mxgetpr(PLHS(2))

      call bargaining_tt_time_watched(%val(time_watched_big_market),%val(time_watched_small_market),%val(bundles),%val(market_share_big_market),%val(market_share_small_market),%val(pre_gamma_big_market),%val(pre_gamma_small_market),%val(market_size),%val(in_package),%val(packages),number_of_channels,simulations_big_market,simulations_small_market,number_of_packages)
                   
      return
      end


        subroutine bargaining_tt_time_watched(time_watched_big_market,time_watched_small_market,bundles,market_share_big_market,market_share_small_market,pre_gamma_big_market,pre_gamma_small_market,market_size,in_package,packages,number_of_channels,simulations_big_market,simulations_small_market,number_of_packages)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations_big_market, simulations_small_market, number_of_channels, number_of_packages
        integer i, j, k, m, n 
        double precision time_watched_big_market(simulations_big_market,number_of_channels), time_watched_small_market(simulations_small_market,number_of_channels)
        double precision market_share_big_market(simulations_big_market,number_of_packages,3), market_share_small_market(simulations_small_market,number_of_packages,3)
        double precision pre_gamma_big_market(simulations_big_market,number_of_channels+1), pre_gamma_small_market(simulations_small_market,number_of_channels+1)
        double precision bundles(number_of_channels,4), market_size(2)
        double precision in_package(number_of_packages,number_of_channels), packages(number_of_packages,5)
        double precision package_gamma_big_market(simulations_big_market,number_of_packages,3), package_gamma_small_market(simulations_small_market,number_of_packages,3)
        double precision pre_time_watched_big_market(simulations_big_market,number_of_channels,number_of_packages), pre_time_watched_small_market(simulations_small_market,number_of_channels,number_of_packages)
        double precision pre_package_gamma_big_market(simulations_big_market,number_of_packages,3,number_of_channels), pre_package_gamma_small_market(simulations_small_market,number_of_packages,3,number_of_channels)

! N themed tiers --> 2^N - 1 packages
do i=1,simulations_big_market
do k=1,number_of_packages
do j=1,number_of_channels

pre_package_gamma_big_market(i,k,1,j) = bundles(j,1)*in_package(k,j)*pre_gamma_big_market(i,j)
pre_package_gamma_big_market(i,k,2,j) = bundles(j,3)*in_package(k,j)*pre_gamma_big_market(i,j)
pre_package_gamma_big_market(i,k,3,j) = bundles(j,4)*in_package(k,j)*pre_gamma_big_market(i,j)

end do
end do
end do

do i=1,simulations_big_market
do k=1,number_of_packages

package_gamma_big_market(i,k,1)=SUM(pre_package_gamma_big_market(i,k,1,:))+pre_gamma_big_market(i,50)
package_gamma_big_market(i,k,2)=SUM(pre_package_gamma_big_market(i,k,2,:))+pre_gamma_big_market(i,50)
package_gamma_big_market(i,k,3)=SUM(pre_package_gamma_big_market(i,k,3,:))+pre_gamma_big_market(i,50)

end do
end do

do i=1,simulations_small_market
do k=1,number_of_packages
do j=1,number_of_channels

pre_package_gamma_small_market(i,k,1,j) = bundles(j,1)*in_package(k,j)*pre_gamma_small_market(i,j)
pre_package_gamma_small_market(i,k,2,j) = bundles(j,3)*in_package(k,j)*pre_gamma_small_market(i,j)
pre_package_gamma_small_market(i,k,3,j) = bundles(j,4)*in_package(k,j)*pre_gamma_small_market(i,j)

end do
end do
end do

do i=1,simulations_small_market
do k=1,number_of_packages

package_gamma_small_market(i,k,1)=SUM(pre_package_gamma_small_market(i,k,1,:))+pre_gamma_small_market(i,50)
package_gamma_small_market(i,k,2)=SUM(pre_package_gamma_small_market(i,k,2,:))+pre_gamma_small_market(i,50)
package_gamma_small_market(i,k,3)=SUM(pre_package_gamma_small_market(i,k,3,:))+pre_gamma_small_market(i,50)

end do
end do

do j=1,number_of_channels
do i=1,simulations_big_market

time_watched_big_market(i,j) = SUM(pre_time_watched_big_market(i,j,:))

end do
end do

do k=1,number_of_packages
do j=1,number_of_channels
do i=1,simulations_small_market
pre_time_watched_small_market(i,j,k) = bundles(j,2)*in_package(k,j)*(pre_gamma_small_market(i,j)/package_gamma_small_market(i,k,1))*market_share_small_market(i,k,1) + bundles(j,3)*in_package(k,j)*(pre_gamma_small_market(i,j)/package_gamma_small_market(i,k,2))*market_share_small_market(i,k,2) + bundles(j,4)*in_package(k,j)*(pre_gamma_small_market(i,j)/package_gamma_small_market(i,k,3))*market_share_small_market(i,k,3)
end do
end do
end do

do j=1,number_of_channels
do i=1,simulations_big_market

time_watched_small_market(i,j) = SUM(pre_time_watched_small_market(i,j,:))

end do
end do


        return
        end

