#include <fintrf.h>
!
!     Matlab call :  [DELTA DELTA_i]=create_DELTA_matrix(alpha_i,income_matrix,predicted_share,same_market_min_max_index,mu_eps);
!
!       alpha_i                                 scalar
!       income_matrix                           number_of_bundles x simulations
!       predicted_share                         number_of_bundles x simulations
!       same_market_min_max_index               number_of_bundles x 2
!       mu_eps                                  scalar

!       Output
!       DELTA                                   number_of_bundles x number_of_bundles


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer DELTA, DELTA_i, alpha_i, income_matrix, predicted_share, same_market_min_max_index, mu_eps
      mwsize mxgetm, mxgetn
      mwsize number_of_bundles, simulations
    
!     RHS parameters
      alpha_i =  mxgetpr(prhs(1)) 
      income_matrix =  mxgetpr(prhs(2)) 
      predicted_share =  mxgetpr(prhs(3)) 
      same_market_min_max_index =  mxgetpr(prhs(4)) 
      mu_eps =  mxgetpr(prhs(5)) 
  
        number_of_bundles=mxgetm(prhs(3))
        simulations=mxgetn(prhs(3))

  
      PLHS(1) = MXCREATEDOUBLEMATRIX(number_of_bundles,number_of_bundles,0)
      DELTA = MXGETPR(PLHS(1))
      PLHS(2) = MXCREATEDOUBLEMATRIX(number_of_bundles,number_of_bundles,0)
      DELTA_i = MXGETPR(PLHS(2))
      
      call create_DELTA_matrix(%val(DELTA),%val(DELTA_i),%val(alpha_i),%val(income_matrix),%val(predicted_share),%val(same_market_min_max_index),%val(mu_eps),number_of_bundles,simulations)
                         
      return
      end


        subroutine create_DELTA_matrix(DELTA,DELTA_i,alpha_i,income_matrix,predicted_share,same_market_min_max_index,mu_eps,number_of_bundles,simulations)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations, number_of_bundles
        integer i, j, k, m, n
        double precision DELTA(number_of_bundles,number_of_bundles), DELTA_i(number_of_bundles,number_of_bundles)
        double precision alpha_i, same_market_min_max_index(number_of_bundles,2)
        double precision income_matrix(number_of_bundles,simulations), predicted_share(number_of_bundles,simulations), mu_eps, pre_computation(simulations), pre_computation_i(simulations)
        integer i_same_market_min_max_index(number_of_bundles,2)
        integer*4, external :: mexPrintf
        character(len=80) :: line

        i_same_market_min_max_index=INT(same_market_min_max_index)

        do i=1,number_of_bundles
          do j=i_same_market_min_max_index(i,1),i_same_market_min_max_index(i,2)
                
                if (i.eq.j) then
                do k=1,simulations
                pre_computation_i(k)=(alpha_i*income_matrix(i,k))*predicted_share(i,k)*(1-predicted_share(i,k))
                pre_computation(k)=predicted_share(i,k)*(1-predicted_share(i,k))
                end do
                DELTA_i(i,j)= SUM(pre_computation_i)/simulations
                DELTA(i,j)= SUM(pre_computation)/simulations
                else
                do k=1,simulations
                pre_computation_i(k)=(alpha_i*income_matrix(i,k))*predicted_share(i,k)*predicted_share(j,k)
                pre_computation(k)=predicted_share(i,k)*predicted_share(j,k)
                end do
                DELTA_i(i,j)= SUM(pre_computation_i)/simulations
                DELTA(i,j)= SUM(pre_computation)/simulations
                end if

     
          end do
        end do
      
    !write(line,*) "tol = ",
    !n = mexPrintf(line//achar(10)) 

      
        return
        end

