#include <fintrf.h>
!
!     Matlab call :  [d_bundle_share d_satellite_share]=create_d_share_matrix(d_v_ij,d_v_ij_satellite,bundle_share,satellite_share,same_market_min_max_index,mu_eps);
!
!       d_v_ij                                  number_of_bundles x simulations
!       d_v_ij_satellite                        number_of_bundles x simulations
!       bundle_share                            number_of_bundles x simulations
!       satellite_share                         number_of_bundles x simulations
!       same_market_min_max_index               number_of_bundles x 2
!       mu_eps                                  scalar

!       Output
!       d_bundle_share                          number_of_bundles x simulations
!       d_satellite_share                       number_of_bundles x simulations


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer d_bundle_share, d_satellite_share, d_v_ij, d_v_ij_satellite, bundle_share, satellite_share, same_market_min_max_index, mu_eps
      mwsize mxgetm, mxgetn
      mwsize number_of_bundles, simulations
    
!     RHS parameters
      d_v_ij =  mxgetpr(prhs(1)) 
      d_v_ij_satellite =  mxgetpr(prhs(2)) 
      bundle_share =  mxgetpr(prhs(3)) 
      satellite_share =  mxgetpr(prhs(4)) 
      same_market_min_max_index =  mxgetpr(prhs(5)) 
      mu_eps =  mxgetpr(prhs(6)) 
  
        number_of_bundles=mxgetm(prhs(3))
        simulations=mxgetn(prhs(3))

  
      PLHS(1) = MXCREATEDOUBLEMATRIX(number_of_bundles,simulations,0)
      d_bundle_share = MXGETPR(PLHS(1))
      PLHS(2) = MXCREATEDOUBLEMATRIX(number_of_bundles,simulations,0)
      d_satellite_share = MXGETPR(PLHS(2))
      
      call create_d_share_matrix(%val(d_bundle_share),%val(d_satellite_share),%val(d_v_ij),%val(d_v_ij_satellite),%val(bundle_share),%val(satellite_share),%val(same_market_min_max_index),%val(mu_eps),number_of_bundles,simulations)
                         
      return
      end


        subroutine create_d_share_matrix(d_bundle_share,d_satellite_share,d_v_ij,d_v_ij_satellite,bundle_share,satellite_share,same_market_min_max_index,mu_eps,number_of_bundles,simulations)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations, number_of_bundles
        integer i, j, k, m, n
        double precision d_bundle_share(number_of_bundles,simulations), d_satellite_share(number_of_bundles,simulations)
        double precision d_v_ij(number_of_bundles,simulations), d_v_ij_satellite(number_of_bundles,simulations), same_market_min_max_index(number_of_bundles,2)
        double precision bundle_share(number_of_bundles,simulations), satellite_share(number_of_bundles,simulations), mu_eps
        double precision pre_deriv(number_of_bundles), pre_deriv_satellite(number_of_bundles)
        integer i_same_market_min_max_index(number_of_bundles,2)
        integer*4, external :: mexPrintf
        character(len=80) :: line

        i_same_market_min_max_index=INT(same_market_min_max_index)

        do i=1,number_of_bundles
                do k=1,simulations

                    do j=i_same_market_min_max_index(i,1),i_same_market_min_max_index(i,2)
                    if (j.ne.i) then
                    pre_deriv(j)=-bundle_share(i,k)*bundle_share(j,k)*d_v_ij(j,k)*mu_eps
                    else
                    pre_deriv(j)=0d0
                    end if
                    pre_deriv_satellite(j)=-satellite_share(i,k)*bundle_share(j,k)*d_v_ij(j,k)*mu_eps
                    end do

                    d_bundle_share(i,k)=d_v_ij(i,k)*bundle_share(i,k)*(1-bundle_share(i,k))*mu_eps + SUM(pre_deriv(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2))) - d_v_ij_satellite(i,k)*bundle_share(i,k)*satellite_share(i,k)*mu_eps
                    d_satellite_share(i,k)=d_v_ij_satellite(i,k)*satellite_share(i,k)*(1-satellite_share(i,k))*mu_eps + SUM(pre_deriv_satellite(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2)))
          
                    pre_deriv(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2))=pre_deriv(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2))*0d0
                    pre_deriv_satellite(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2))=pre_deriv_satellite(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2))*0d0

     
          end do
        end do
      
    !write(line,*) "tol = ",
    !n = mexPrintf(line//achar(10)) 

      
        return
        end

