#include <fintrf.h>
!
!     Matlab call :  [deltas]=delta_contraction(last_deltas,v_ij,v_ij_satellite,price_interactions,price_interactions_satellite,X,outside_good_utility,same_market_min_max_index,same_dma_min_max_index,sys_size_vector,bundle_market_shares,satellite_share,mu_eps);
!
!       utility_demographics_matrices               (number_of_bundles) x simulations x 10
!       hh_viewing                                  (number_of_bundles) x simulations x number_of_channels

!       Output
!       deltas                                      number_of_bundles x 2


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer deltas, last_deltas, v_ij, v_ij_satellite, price_interactions, price_interactions_satellite, X, same_market_min_max_index,same_dma_min_max_index, outside_good_utility
      mwpointer sys_size_vector, bundles_market_shares, satellite_share, mu_eps
      mwsize mxgetm, mxgetn
      mwsize number_of_channels, number_of_bundles, simulations
    
!     RHS parameters
      
      last_deltas=mxgetpr(prhs(1)) 
      v_ij =  mxgetpr(prhs(2)) 
      v_ij_satellite =  mxgetpr(prhs(3)) 
      price_interactions =  mxgetpr(prhs(4))
      price_interactions_satellite =  mxgetpr(prhs(5))
      X = mxgetpr(prhs(6)) 
      outside_good_utility =  mxgetpr(prhs(7)) 
      same_market_min_max_index = mxgetpr(prhs(8)) 
      same_dma_min_max_index = mxgetpr(prhs(9)) 
      sys_size_vector =  mxgetpr(prhs(10)) 
      bundles_market_shares =  mxgetpr(prhs(11)) 
      satellite_share =  mxgetpr(prhs(12)) 
      mu_eps =  mxgetpr(prhs(13)) 
   
        number_of_bundles=mxgetm(prhs(1))
        simulations=mxgetn(prhs(2))

  
      PLHS(1) = MXCREATEDOUBLEMATRIX(number_of_bundles,2,0)
      deltas = MXGETPR(PLHS(1))

      
      call delta_contraction(%val(deltas),%val(last_deltas),%val(v_ij),%val(v_ij_satellite),%val(price_interactions),%val(price_interactions_satellite),%val(X),%val(outside_good_utility),%val(same_market_min_max_index),%val(same_dma_min_max_index),%val(sys_size_vector),%val(bundles_market_shares),%val(satellite_share),%val(mu_eps),number_of_bundles,simulations)
                         
      return
      end


        subroutine delta_contraction(deltas,last_deltas,v_ij,v_ij_satellite,price_interactions,price_interactions_satellite,X,outside_good_utility,same_market_min_max_index,same_dma_min_max_index,sys_size_vector,bundles_market_shares,satellite_share,mu_eps,number_of_bundles,simulations)
        include 'link_fnl_static.h'
        implicit none

        mwsize simulations, number_of_bundles
        integer i, j, k, m, n
        double precision deltas(number_of_bundles,2), last_deltas(number_of_bundles,2), v_ij(number_of_bundles,simulations), v_ij_satellite(number_of_bundles,simulations)
        double precision price_interactions(number_of_bundles,simulations), price_interactions_satellite(number_of_bundles,simulations)
        double precision X(number_of_bundles,6), outside_good_utility(number_of_bundles,simulations), new_deltas(number_of_bundles,2), mu_eps
        double precision same_market_min_max_index(number_of_bundles,2), sys_size_vector(number_of_bundles), bundles_market_shares(number_of_bundles), satellite_share(number_of_bundles), same_dma_min_max_index(number_of_bundles,2)
        double precision deltas_tolerance, numerator(number_of_bundles,simulations), satellite_utility(number_of_bundles,simulations), denominator(number_of_bundles,simulations)
        double precision predicted_hh_cable_share(number_of_bundles,simulations), predicted_hh_satellite_share(number_of_bundles,simulations), predicted_cable_share(number_of_bundles), predicted_satellite_share(number_of_bundles), pre_predicted_satellite_share(number_of_bundles)
        integer i_same_market_min_max_index(number_of_bundles,2), i_same_dma_min_max_index(number_of_bundles,2)
        integer*4, external :: mexPrintf
        character(len=80) :: line


        i_same_market_min_max_index=INT(same_market_min_max_index)
        i_same_dma_min_max_index=INT(same_dma_min_max_index)

        deltas=last_deltas
        m=1

        deltas_tolerance=1d0
        do while (deltas_tolerance>1d-4)
        !do while (m<350)

            do k=1,simulations
                do i=1,number_of_bundles
                
            numerator(i,k) =  (v_ij(i,k)*price_interactions(i,k))*deltas(i,1)
            satellite_utility(i,k) =  (v_ij_satellite(i,k)*price_interactions_satellite(i,k))*deltas(i,2)
            end do

            end do


            do k=1,simulations
            do i=1,number_of_bundles
                    
            denominator(i,k) = SUM(numerator(i_same_market_min_max_index(i,1):i_same_market_min_max_index(i,2),k))+outside_good_utility(i,k)+satellite_utility(i,k)
            predicted_hh_cable_share(i,k)= numerator(i,k)/denominator(i,k)
            predicted_hh_satellite_share(i,k)= satellite_utility(i,k)/denominator(i,k)

            end do
            end do

      

        predicted_cable_share=SUM(predicted_hh_cable_share,2)/DBLE(simulations)
        pre_predicted_satellite_share=SUM(predicted_hh_satellite_share,2)/DBLE(simulations)

        do i=1,number_of_bundles
        predicted_satellite_share(i)=DOT_PRODUCT(sys_size_vector(i_same_dma_min_max_index(i,1):i_same_dma_min_max_index(i,2)),pre_predicted_satellite_share(i_same_dma_min_max_index(i,1):i_same_dma_min_max_index(i,2)))

        if ((deltas(i,1).ge.1d-300).AND.(deltas(i,2).ge.1d-300)) then
        new_deltas(i,1)=deltas(i,1)*(bundles_market_shares(i)/predicted_cable_share(i))
        new_deltas(i,2)=deltas(i,2)*(satellite_share(i)/predicted_satellite_share(i))
        end if
        end do



        deltas_tolerance=MAXVAL(ABS(log(new_deltas(:,1))-log(deltas(:,1))))
        deltas=new_deltas
        m=m+1

    !write(line,*) "tol = ",deltas_tolerance
    !n = mexPrintf(line//achar(10)) 

        end do

        return
        end

