#include <fintrf.h>
!
!     Matlab call :  [predicted_mean_viewership_by_demographic channel_viewing_vector channel_viewing_weights]=group_stats(utility_demographics_matrices,hh_viewing,homespassed_matrix);
!
!       utility_demographics_matrices               (number_of_bundles) x simulations x 10
!       hh_viewing                                  (number_of_bundles) x simulations x number_of_channels
!       homespassed_matrix                          number_of_bundles x simulations

!       Output
!       predicted_mean_viewership_by_demographic    10 x number_of_channels             
!       channel_viewing_vector                      number_of_bundles*simulations x number_of_channels
!       channel_viewing_weights                      number_of_bundles*simulations x number_of_channels


!     Gateway function to import matlab arrays, define outputs, error checking, and call numerical subroutines

      subroutine mexfunction(nlhs,plhs,nrhs,prhs)
      implicit none
      integer nlhs,nrhs
      mwpointer plhs(*),prhs(*)
      mwpointer mxgetpr, mxcreatecellarray, mxcreatedoublematrix, mxcreatedoublescalar, mxCreateNumericArray, mxClassIDFromClassName
      mwpointer utility_demographics_matrices, hh_viewing, predicted_mean_viewership_by_demographic, channel_viewership_vector, homespassed_matrix, channel_viewership_weights
      mwsize mxgetm, mxgetn
      mwsize number_of_channels, number_of_bundles, simulations
    
!     RHS parameters
      
      utility_demographics_matrices=mxgetpr(prhs(1)) 
      hh_viewing=mxgetpr(prhs(2)) 
      homespassed_matrix=mxgetpr(prhs(3)) 
         
        simulations=mxgetn(prhs(1))/10
        number_of_channels=mxgetn(prhs(2))/simulations      
        number_of_bundles=mxgetm(prhs(1))
  
      PLHS(1) = MXCREATEDOUBLEMATRIX(10,number_of_channels,0)
      predicted_mean_viewership_by_demographic = MXGETPR(PLHS(1))

      PLHS(2) = MXCREATEDOUBLEMATRIX(number_of_bundles*simulations,number_of_channels,0)
      channel_viewership_vector = MXGETPR(PLHS(2))

      PLHS(3) = MXCREATEDOUBLEMATRIX(number_of_bundles*simulations,number_of_channels,0)
      channel_viewership_weights = MXGETPR(PLHS(3))

      call group_stats(%val(predicted_mean_viewership_by_demographic),%val(channel_viewership_vector),%val(channel_viewership_weights),%val(hh_viewing),%val(utility_demographics_matrices),%val(homespassed_matrix),number_of_bundles,simulations,number_of_channels)
                               
      return
      end


        subroutine group_stats(predicted_mean_viewership_by_demographic,channel_viewership_vector,channel_viewership_weights,hh_viewing,utility_demographics_matrices,homespassed_matrix,number_of_bundles,simulations,number_of_channels)
        include 'link_fnl_static.h'
        implicit none

        mwsize number_of_channels, simulations, number_of_bundles
        integer i, j, k, m, n
        double precision predicted_mean_viewership_by_demographic(10,number_of_channels), channel_viewership_vector(number_of_bundles*simulations,number_of_channels), channel_viewership_weights(number_of_bundles*simulations,number_of_channels)
        double precision utility_demographics_matrices(number_of_bundles,simulations,10), hh_viewing(number_of_bundles,simulations,number_of_channels), homespassed_matrix(number_of_bundles,simulations)
        double precision pre_sum(simulations), pre_pre_sum(number_of_bundles), pre_pre_denom(number_of_bundles), pre_denom(simulations)
        
            do i=1,number_of_channels


            do k=1,10
                
                pre_denom=SUM(utility_demographics_matrices(:,:,k))

                do m=1,simulations
                

!$OMP PARALLEL SHARED(utility_demographics_matrices,hh_viewing,homespassed_matrix,pre_pre_sum,pre_pre_denom,number_of_bundles) PRIVATE(n), NUM_THREADS(4)
!$OMP DO SCHEDULE(DYNAMIC,3000)

                do n=1,number_of_bundles
                if (utility_demographics_matrices(n,m,k).eq.1) then
                pre_pre_sum(n)=hh_viewing(n,m,i)*24d0*7d0*homespassed_matrix(n,m)
                pre_pre_denom(n)=homespassed_matrix(n,m)
                else
                pre_pre_sum(n)=0d0
                pre_pre_denom(n)=0d0
                end if
                end do


!$OMP END DO NOWAIT

!$OMP END PARALLEL

                pre_sum(m)=SUM(pre_pre_sum)
                pre_denom(m) = SUM(pre_pre_denom)
                end do

              predicted_mean_viewership_by_demographic(k,i)=SUM(pre_sum) /  SUM(pre_denom)

                end do 

                
                j=0
                do m=1,simulations
                 do n=1,number_of_bundles
                   j=j+1
                    channel_viewership_vector(j,i)=hh_viewing(n,m,i)*24d0*7d0
                    channel_viewership_weights(j,i)=homespassed_matrix(n,m)
                  end do
                end do
                
            end do

        return
        end

